/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceForm.hpp"
#include "BinaryData.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceForm::
DeviceForm (int          id,
            int          iCapabilities,
            BinaryData  *data,
            HardCopyCap *hcInfo)
{
   iXOverride_d    = 0;
   iYOverride_d    = 0;
   id_d            = id;
   iCapabilities_d = iCapabilities;
   data_d          = data;
   hcInfo_d        = hcInfo;

   if (hcInfo)
   {
      hcInfo->setOwner (this);
   }
}

DeviceForm::
~DeviceForm ()
{
   delete data_d;
   delete hcInfo_d;
}

char * DeviceForm::
getName (int id)
{
   return StringResource::getString (StringResource::STRINGGROUP_FORMS, id);
}

char * DeviceForm::
getName ()
{
   return StringResource::getString (StringResource::STRINGGROUP_FORMS, id_d);
}

bool DeviceForm::
isID (int id)
{
   return id == id_d;
}

int DeviceForm::
getID ()
{
   return id_d;
}

bool DeviceForm::
hasCapability (int iFlag)
{
   return 0 != (iFlag & iCapabilities_d);
}

int DeviceForm::
getCapabilities ()
{
   return iCapabilities_d;
}

BinaryData * DeviceForm::
getData ()
{
   return data_d;
}

HardCopyCap * DeviceForm::
getHardCopyCap ()
{
   return hcInfo_d;
}

void DeviceForm::
associateWith (DeviceResolution *pResolution)
{
   hcInfo_d->associateWith (pResolution);
}

DeviceForm * DeviceForm::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszFormNames[x - FORM_UNLISTED])

static char *apszFormNames[] = {
   "FORM_UNLISTED",
   "FORM_NONE",
   "FORM_USER_DEFINED",
   "FORM_TEST",
   "FORM_TEST2",
   "FORM_LEDGER",
   "FORM_LEGAL",
   "FORM_LEGAL_EXTRA",
   "FORM_LETTER",
   "FORM_LETTER_PLUS",
   "FORM_LETTER_EXTRA",
   "FORM_HALF_LETTER",
   "FORM_EXECUTIVE",
   "FORM_STATEMENT",
   "FORM_TABLOID",
   "FORM_UNIVERSAL",
   "FORM_FOOLSCAP",
   "FORM_QUARTO",
   "FORM_FOLIO",
   "FORM_FOLIO_SP",
   "FORM_ROC_8K",
   "FORM_ROC_16K",
   "FORM_GOVERNMENT_LETTER",
   "FORM_GOVERNMENT_LEGAL",
   "FORM_EDP",
   "FORM_EUROPEAN_EDP",
   "FORM_PRC_16K",
   "FORM_PRC_32K",
   "FORM_DAI_PA_KAI",
   "FORM_JURRO_KU_KAI",
   "FORM_PA_KAI",
   "FORM_WIDE",
   "FORM_LETTER_WIDE",
   "FORM_FOOLSCAP_WIDE",
   "FORM_A3_WIDE",
   "FORM_A4_WIDE",
   "FORM_WIDE_FORMAT",
   "FORM_10_X_11",
   "FORM_11_X_12",
   "FORM_11_X_15",
   "FORM_12_X_19",
   "FORM_15_X_11",
   "FORM_5_X_7",
   "FORM_3_X_5_CARD",
   "FORM_4_X_6_CARD",
   "FORM_5_X_8_CARD",
   "FORM_8_X_10_CARD",
   "FORM_A6_CARD",
   "FORM_CARD_148",
   "FORM_HAGAKI_CARD",
   "FORM_OUFUKU_CARD",
   "FORM_POSTCARD",
   "FORM_INDEX_4_X_6_EXT",
   "FORM_A2_ENVELOPE",
   "FORM_C5_ENVELOPE",
   "FORM_C6_ENVELOPE",
   "FORM_C7_ENVELOPE",
   "FORM_C9_ENVELOPE",
   "FORM_C10_ENVELOPE",
   "FORM_D5_ENVELOPE",
   "FORM_DL_ENVELOPE",
   "FORM_ENVELOPE_132_X_220",
   "FORM_ENVELOPE_6_X_9",
   "FORM_ENVELOPE_7_X_9",
   "FORM_ENVELOPE_6_1_2",
   "FORM_ENVELOPE_9_X_11",
   "FORM_ENVELOPE_10_X_13",
   "FORM_ENVELOPE_10_X_14",
   "FORM_ENVELOPE_10_X_15",
   "FORM_MONARCH_ENVELOPE",
   "FORM_PERSONAL_ENVELOPE",
   "FORM_NUMBER_9_ENVELOPE",
   "FORM_NUMBER_10_ENVELOPE",
   "FORM_NUMBER_11_ENVELOPE",
   "FORM_NUMBER_12_ENVELOPE",
   "FORM_NUMBER_14_ENVELOPE",
   "FORM_ITALIAN_ENVELOPE",
   "FORM_POSTFIX_ENVELOPE",
   "FORM_INVITE_ENVELOPE",
   "FORM_CHOU2_ENVELOPE",
   "FORM_CHOU3_ENVELOPE",
   "FORM_CHOU4_ENVELOPE",
   "FORM_KAHU_ENVELOPE",
   "FORM_KAKU2_ENVELOPE",
   "FORM_YOU4_ENVELOPE",
   "FORM_PRC1_ENVELOPE",
   "FORM_PRC2_ENVELOPE",
   "FORM_PRC3_ENVELOPE",
   "FORM_PRC4_ENVELOPE",
   "FORM_PRC5_ENVELOPE",
   "FORM_PRC6_ENVELOPE",
   "FORM_PRC7_ENVELOPE",
   "FORM_PRC8_ENVELOPE",
   "FORM_PRC9_ENVELOPE",
   "FORM_PRC10_ENVELOPE",
   "FORM_DISK_LABELS",
   "FORM_EURO_LABELS",
   "FORM_SHIPPING_LABELS",
   "FORM_STANDARD_LABELS_CLEAR",
   "FORM_STANDARD_LABELS_WHITE",
   "FORM_2A0",
   "FORM_4A0",
   "FORM_A0",
   "FORM_A1",
   "FORM_A2",
   "FORM_A3",
   "FORM_A3_EXTRA",
   "FORM_A4",
   "FORM_A4_EXTRA",
   "FORM_A4_TAB",
   "FORM_A5",
   "FORM_A5_EXTRA",
   "FORM_A6",
   "FORM_A7",
   "FORM_A8",
   "FORM_A9",
   "FORM_A10",
   "FORM_B_PLUS",
   "FORM_B0",
   "FORM_B1",
   "FORM_B2",
   "FORM_B3",
   "FORM_B4",
   "FORM_B5",
   "FORM_B5_EXTRA",
   "FORM_B6",
   "FORM_B6_C4",
   "FORM_B7",
   "FORM_B8",
   "FORM_B9",
   "FORM_B10",
   "FORM_JIS_B0",
   "FORM_JIS_B1",
   "FORM_JIS_B2",
   "FORM_JIS_B3",
   "FORM_JIS_B4",
   "FORM_JIS_B5",
   "FORM_JIS_B6",
   "FORM_JIS_B7",
   "FORM_JIS_B8",
   "FORM_JIS_B9",
   "FORM_JIS_B10",
   "FORM_JIS_EXEC",
   "FORM_C0",
   "FORM_C1",
   "FORM_C2",
   "FORM_C3",
   "FORM_C4",
   "FORM_C5",
   "FORM_C6",
   "FORM_C6_C5",
   "FORM_C7",
   "FORM_C7_C6",
   "FORM_C8",
   "FORM_C9",
   "FORM_C10",
   "FORM_E1",
   "FORM_RA0",
   "FORM_RA1",
   "FORM_RA2",
   "FORM_SRA0",
   "FORM_SRA1",
   "FORM_SRA2",
   "FORM_US_FANFOLD",
   "FORM_FANFOLD_1",
   "FORM_FANFOLD_2",
   "FORM_FANFOLD_3",
   "FORM_FANFOLD_4",
   "FORM_FANFOLD_5",
   "FORM_GERMAN_LEGAL_FANFOLD",
   "FORM_EUROPEAN_FANFOLD",
   "FORM_PANORAMIC",
   "FORM_PHOTO_100_150",
   "FORM_PHOTO_200_300",
   "FORM_PHOTO_4_6",
   "FORM_PLOTTER_SIZE_A",
   "FORM_PLOTTER_SIZE_B",
   "FORM_PLOTTER_SIZE_C",
   "FORM_PLOTTER_SIZE_D",
   "FORM_PLOTTER_SIZE_E",
   "FORM_PLOTTER_SIZE_F",
   "FORM_US_A_ARCHITECTURAL",
   "FORM_US_B_ARCHITECTURAL",
   "FORM_US_C_ARCHITECTURAL",
   "FORM_US_D_ARCHITECTURAL",
   "FORM_US_E_ARCHITECTURAL",
   "FORM_SUPER_A3_B",
   "FORM_SUPER_A",
   "FORM_SUPER_B",
   "FORM_ROLL_76_2MM",
   "FORM_ROLL_69_5MM"
};

int DeviceForm::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      FORM_10_X_11,
      FORM_11_X_12,
      FORM_11_X_15,
      FORM_12_X_19,
      FORM_15_X_11,
      FORM_2A0,
      FORM_3_X_5_CARD,
      FORM_4A0,
      FORM_4_X_6_CARD,
      FORM_5_X_7,
      FORM_5_X_8_CARD,
      FORM_8_X_10_CARD,
      FORM_A0,
      FORM_A1,
      FORM_A10,
      FORM_A2,
      FORM_A2_ENVELOPE,
      FORM_A3,
      FORM_A3_EXTRA,
      FORM_A3_WIDE,
      FORM_A4,
      FORM_A4_EXTRA,
      FORM_A4_TAB,
      FORM_A4_WIDE,
      FORM_A5,
      FORM_A5_EXTRA,
      FORM_A6,
      FORM_A6_CARD,
      FORM_A7,
      FORM_A8,
      FORM_A9,
      FORM_B0,
      FORM_B1,
      FORM_B10,
      FORM_B2,
      FORM_B3,
      FORM_B4,
      FORM_B5,
      FORM_B5_EXTRA,
      FORM_B6,
      FORM_B6_C4,
      FORM_B7,
      FORM_B8,
      FORM_B9,
      FORM_B_PLUS,
      FORM_C0,
      FORM_C1,
      FORM_C10,
      FORM_C10_ENVELOPE,
      FORM_C2,
      FORM_C3,
      FORM_C4,
      FORM_C5,
      FORM_C5_ENVELOPE,
      FORM_C6,
      FORM_C6_C5,
      FORM_C6_ENVELOPE,
      FORM_C7,
      FORM_C7_C6,
      FORM_C7_ENVELOPE,
      FORM_C8,
      FORM_C9,
      FORM_C9_ENVELOPE,
      FORM_CARD_148,
      FORM_CHOU2_ENVELOPE,
      FORM_CHOU3_ENVELOPE,
      FORM_CHOU4_ENVELOPE,
      FORM_D5_ENVELOPE,
      FORM_DAI_PA_KAI,
      FORM_DISK_LABELS,
      FORM_DL_ENVELOPE,
      FORM_E1,
      FORM_EDP,
      FORM_ENVELOPE_10_X_13,
      FORM_ENVELOPE_10_X_14,
      FORM_ENVELOPE_10_X_15,
      FORM_ENVELOPE_132_X_220,
      FORM_ENVELOPE_6_1_2,
      FORM_ENVELOPE_6_X_9,
      FORM_ENVELOPE_7_X_9,
      FORM_ENVELOPE_9_X_11,
      FORM_EUROPEAN_EDP,
      FORM_EUROPEAN_FANFOLD,
      FORM_EURO_LABELS,
      FORM_EXECUTIVE,
      FORM_FANFOLD_1,
      FORM_FANFOLD_2,
      FORM_FANFOLD_3,
      FORM_FANFOLD_4,
      FORM_FANFOLD_5,
      FORM_FOLIO,
      FORM_FOLIO_SP,
      FORM_FOOLSCAP,
      FORM_FOOLSCAP_WIDE,
      FORM_GERMAN_LEGAL_FANFOLD,
      FORM_GOVERNMENT_LEGAL,
      FORM_GOVERNMENT_LETTER,
      FORM_HAGAKI_CARD,
      FORM_HALF_LETTER,
      FORM_INDEX_4_X_6_EXT,
      FORM_INVITE_ENVELOPE,
      FORM_ITALIAN_ENVELOPE,
      FORM_JIS_B0,
      FORM_JIS_B1,
      FORM_JIS_B10,
      FORM_JIS_B2,
      FORM_JIS_B3,
      FORM_JIS_B4,
      FORM_JIS_B5,
      FORM_JIS_B6,
      FORM_JIS_B7,
      FORM_JIS_B8,
      FORM_JIS_B9,
      FORM_JIS_EXEC,
      FORM_JURRO_KU_KAI,
      FORM_KAHU_ENVELOPE,
      FORM_KAKU2_ENVELOPE,
      FORM_LEDGER,
      FORM_LEGAL,
      FORM_LEGAL_EXTRA,
      FORM_LETTER,
      FORM_LETTER_EXTRA,
      FORM_LETTER_PLUS,
      FORM_LETTER_WIDE,
      FORM_MONARCH_ENVELOPE,
      FORM_NONE,
      FORM_NUMBER_10_ENVELOPE,
      FORM_NUMBER_11_ENVELOPE,
      FORM_NUMBER_12_ENVELOPE,
      FORM_NUMBER_14_ENVELOPE,
      FORM_NUMBER_9_ENVELOPE,
      FORM_OUFUKU_CARD,
      FORM_PANORAMIC,
      FORM_PA_KAI,
      FORM_PERSONAL_ENVELOPE,
      FORM_PHOTO_100_150,
      FORM_PHOTO_200_300,
      FORM_PHOTO_4_6,
      FORM_PLOTTER_SIZE_A,
      FORM_PLOTTER_SIZE_B,
      FORM_PLOTTER_SIZE_C,
      FORM_PLOTTER_SIZE_D,
      FORM_PLOTTER_SIZE_E,
      FORM_PLOTTER_SIZE_F,
      FORM_POSTCARD,
      FORM_POSTFIX_ENVELOPE,
      FORM_PRC10_ENVELOPE,
      FORM_PRC1_ENVELOPE,
      FORM_PRC2_ENVELOPE,
      FORM_PRC3_ENVELOPE,
      FORM_PRC4_ENVELOPE,
      FORM_PRC5_ENVELOPE,
      FORM_PRC6_ENVELOPE,
      FORM_PRC7_ENVELOPE,
      FORM_PRC8_ENVELOPE,
      FORM_PRC9_ENVELOPE,
      FORM_PRC_16K,
      FORM_PRC_32K,
      FORM_QUARTO,
      FORM_RA0,
      FORM_RA1,
      FORM_RA2,
      FORM_ROC_16K,
      FORM_ROC_8K,
      FORM_ROLL_69_5MM,
      FORM_ROLL_76_2MM,
      FORM_SHIPPING_LABELS,
      FORM_SRA0,
      FORM_SRA1,
      FORM_SRA2,
      FORM_STANDARD_LABELS_CLEAR,
      FORM_STANDARD_LABELS_WHITE,
      FORM_STATEMENT,
      FORM_SUPER_A,
      FORM_SUPER_A3_B,
      FORM_SUPER_B,
      FORM_TABLOID,
      FORM_TEST,
      FORM_TEST2,
      FORM_UNIVERSAL,
      FORM_UNLISTED,
      FORM_USER_DEFINED,
      FORM_US_A_ARCHITECTURAL,
      FORM_US_B_ARCHITECTURAL,
      FORM_US_C_ARCHITECTURAL,
      FORM_US_D_ARCHITECTURAL,
      FORM_US_E_ARCHITECTURAL,
      FORM_US_FANFOLD,
      FORM_WIDE,
      FORM_WIDE_FORMAT,
      FORM_YOU4_ENVELOPE
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return FORM_UNLISTED;
}

char * DeviceForm::
IDToName (int id)
{
   id -= FORM_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszFormNames)
      )
      return apszFormNames[id];
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + FORM_UNLISTED);

      return achUnknown;
   }
}

typedef struct _FormSize {
   int cx;
   int cy;
} FORMSIZE, *PFORMSIZE;

static FORMSIZE vaFormSizes[] = {
   /* FORM_USER_DEFINED          */ {       0,        0 },
   /* FORM_TEST                  */ {   69500,    76200 },
   /* FORM_TEST2                 */ {       0,        0 },
   /* FORM_LEDGER                */ {  431800,   279400 },
   /* FORM_LEGAL                 */ {  215900,   355600 },
   /* FORM_LEGAL_EXTRA           */ {  241300,   381000 },
   /* FORM_LETTER                */ {  215900,   279400 },
   /* FORM_LETTER_PLUS           */ {  215900,   322326 },
   /* FORM_LETTER_EXTRA          */ {  241300,   304800 },
   /* FORM_HALF_LETTER           */ {  139690,   215970 },
   /* FORM_EXECUTIVE             */ {  184150,   266700 },
   /* FORM_STATEMENT             */ {  139700,   215900 },
   /* FORM_TABLOID               */ {  279400,   431800 },
   /* FORM_UNIVERSAL             */ {  297040,   431800 },
   /* FORM_FOOLSCAP              */ {  203200,   330200 },
   /* FORM_QUARTO                */ {  215900,   275082 },
   /* FORM_FOLIO                 */ {  215900,   330000 },
   /* FORM_FOLIO_SP              */ {  215000,   315000 },
   /* FORM_ROC_8K                */ {  273050,   393700 },
   /* FORM_ROC_16K               */ {  196850,   273050 },
   /* FORM_GOVERNMENT_LETTER     */ {  203200,   254000 },
   /* FORM_GOVERNMENT_LEGAL      */ {  203200,   330200 },
   /* FORM_EDP                   */ {  279400,   355600 },
   /* FORM_EUROPEAN_EDP          */ {  294000,   355600 },
   /* FORM_PRC_16K               */ {  146000,   215000 },
   /* FORM_PRC_32K               */ {   97000,   151000 },
   /* FORM_DAI_PA_KAI            */ {  275000,   395000 },
   /* FORM_JURRO_KU_KAI          */ {  198000,   275000 },
   /* FORM_PA_KAI                */ {  267000,   389000 },
   /* FORM_WIDE                  */ {  345000,   279400 },
   /* FORM_LETTER_WIDE           */ {  228400,   337800 },
   /* FORM_FOOLSCAP_WIDE         */ {  215900,   330200 },
   /* FORM_A3_WIDE               */ {  330200,   482600 },
   /* FORM_A4_WIDE               */ {  223500,   355600 },
   /* FORM_WIDE_FORMAT           */ {  762000,  1066800 },
   /* FORM_10_X_11               */ {  254000,   279400 },
   /* FORM_11_X_12               */ {  279400,   304800 },
   /* FORM_11_X_15               */ {  279400,   381000 },
   /* FORM_12_X_19               */ {  304800,   482600 },
   /* FORM_15_X_11               */ {  381000,   279400 },
   /* FORM_5_X_7                 */ {  127000,   177800 },
   /* FORM_3_X_5_CARD            */ {   76200,   127000 },
   /* FORM_4_X_6_CARD            */ {  101600,   152400 },
   /* FORM_5_X_8_CARD            */ {  127000,   203200 },
   /* FORM_8_X_10_CARD           */ {  203200,   254000 },
   /* FORM_A6_CARD               */ {  104980,   148090 },
   /* FORM_CARD_148              */ {  148000,   105000 },
   /* FORM_HAGAKI_CARD           */ {  100000,   148000 },
   /* FORM_OUFUKU_CARD           */ {  148000,   200000 },
   /* FORM_POSTCARD              */ {   96970,   147030 },
   /* FORM_INDEX_4_X_6_EXT       */ {  152400,   203200 },
   /* FORM_A2_ENVELOPE           */ {  111125,   146050 },
   /* FORM_C5_ENVELOPE           */ {  162000,   229000 },
   /* FORM_C6_ENVELOPE           */ {  114000,   162000 },
   /* FORM_C7_ENVELOPE           */ {   98400,   190500 },
   /* FORM_C9_ENVELOPE           */ {   98400,   225400 },
   /* FORM_C10_ENVELOPE          */ {  104700,   241300 },
   /* FORM_D5_ENVELOPE           */ {  176000,   250000 },
   /* FORM_DL_ENVELOPE           */ {  110000,   220000 },
   /* FORM_ENVELOPE_132_X_220    */ {  132000,   220000 },
   /* FORM_ENVELOPE_6_X_9        */ {  152400,   228600 },
   /* FORM_ENVELOPE_7_X_9        */ {  177800,   228600 },
   /* FORM_ENVELOPE_6_1_2        */ {  165100,    92070 },
   /* FORM_ENVELOPE_9_X_11       */ {  228600,   279400 },
   /* FORM_ENVELOPE_10_X_13      */ {  254000,   330200 },
   /* FORM_ENVELOPE_10_X_14      */ {  254000,   355600 },
   /* FORM_ENVELOPE_10_X_15      */ {  254000,   381000 },
   /* FORM_MONARCH_ENVELOPE      */ {   98425,   190500 },
   /* FORM_PERSONAL_ENVELOPE     */ {   92075,   165100 },
   /* FORM_NUMBER_9_ENVELOPE     */ {   98425,   225425 },
   /* FORM_NUMBER_10_ENVELOPE    */ {  104775,   241300 },
   /* FORM_NUMBER_11_ENVELOPE    */ {  114300,   263525 },
   /* FORM_NUMBER_12_ENVELOPE    */ {  120650,   279400 },
   /* FORM_NUMBER_14_ENVELOPE    */ {  127000,   292100 },
   /* FORM_ITALIAN_ENVELOPE      */ {  100000,   230000 },
   /* FORM_POSTFIX_ENVELOPE      */ {  114000,   229000 },
   /* FORM_INVITE_ENVELOPE       */ {  220000,   220000 },
   /* FORM_CHOU2_ENVELOPE        */ {  111100,   146000 },
   /* FORM_CHOU3_ENVELOPE        */ {  120000,   235000 },
   /* FORM_CHOU4_ENVELOPE        */ {   90000,   205000 },
   /* FORM_KAHU_ENVELOPE         */ {  240000,   322100 },
   /* FORM_KAKU2_ENVELOPE        */ {  240000,   332000 },
   /* FORM_YOU4_ENVELOPE         */ {  105000,   235000 },
   /* FORM_PRC1_ENVELOPE         */ {  102000,   165000 },
   /* FORM_PRC2_ENVELOPE         */ {  102000,   176000 },
   /* FORM_PRC3_ENVELOPE         */ {  125000,   176000 },
   /* FORM_PRC4_ENVELOPE         */ {  110000,   208000 },
   /* FORM_PRC5_ENVELOPE         */ {  110000,   220000 },
   /* FORM_PRC6_ENVELOPE         */ {  120000,   320000 },
   /* FORM_PRC7_ENVELOPE         */ {  160000,   230000 },
   /* FORM_PRC8_ENVELOPE         */ {  120000,   309000 },
   /* FORM_PRC9_ENVELOPE         */ {  229000,   324000 },
   /* FORM_PRC10_ENVELOPE        */ {  324000,   458000 },
   /* FORM_DISK_LABELS           */ {   54000,    70000 },
   /* FORM_EURO_LABELS           */ {   36000,    89000 },
   /* FORM_SHIPPING_LABELS       */ {   54000,   101000 },
   /* FORM_STANDARD_LABELS_CLEAR */ {   28000,    89000 },
   /* FORM_STANDARD_LABELS_WHITE */ {   28000,    89000 },
   /* FORM_2A0                   */ { 1189000,  1682000 },
   /* FORM_4A0                   */ { 1682000,  2378000 },
   /* FORM_A0                    */ {  841000,  1189000 },
   /* FORM_A1                    */ {  594000,   841000 },
   /* FORM_A2                    */ {  420000,   594000 },
   /* FORM_A3                    */ {  297000,   420000 },
   /* FORM_A3_EXTRA              */ {  322000,   445000 },
   /* FORM_A4                    */ {  210000,   297000 },
   /* FORM_A4_EXTRA              */ {  235500,   322300 },
   /* FORM_A4_TAB                */ {  225000,   297000 },
   /* FORM_A5                    */ {  148000,   210000 },
   /* FORM_A5_EXTRA              */ {  174000,   235000 },
   /* FORM_A6                    */ {  105000,   148000 },
   /* FORM_A7                    */ {   74000,   105000 },
   /* FORM_A8                    */ {   52000,    74000 },
   /* FORM_A9                    */ {   37000,    52000 },
   /* FORM_A10                   */ {   26000,    37000 },
   /* FORM_B_PLUS                */ {  304800,   486918 },
   /* FORM_B0                    */ { 1000000,  1414000 },
   /* FORM_B1                    */ {  707000,  1000000 },
   /* FORM_B2                    */ {  500000,   707000 },
   /* FORM_B3                    */ {  353000,   500000 },
   /* FORM_B4                    */ {  250000,   353000 },
   /* FORM_B5                    */ {  176000,   250000 },
   /* FORM_B5_EXTRA              */ {  201000,   276000 },
   /* FORM_B6                    */ {  125000,   176000 },
   /* FORM_B6_C4                 */ {  125000,   324000 },
   /* FORM_B7                    */ {   88000,   125000 },
   /* FORM_B8                    */ {   62000,    88000 },
   /* FORM_B9                    */ {   44000,    62000 },
   /* FORM_B10                   */ {   31000,    44000 },
   /* FORM_JIS_B0                */ { 1030000,  1456000 },
   /* FORM_JIS_B1                */ {  728000,  1030000 },
   /* FORM_JIS_B2                */ {  515000,   728000 },
   /* FORM_JIS_B3                */ {  364000,   515000 },
   /* FORM_JIS_B4                */ {  257000,   364000 },
   /* FORM_JIS_B5                */ {  182000,   257000 },
   /* FORM_JIS_B6                */ {  128000,   182000 },
   /* FORM_JIS_B7                */ {   91000,   128000 },
   /* FORM_JIS_B8                */ {   64000,    91000 },
   /* FORM_JIS_B9                */ {   45000,    64000 },
   /* FORM_JIS_B10               */ {   32000,    45000 },
   /* FORM_JIS_EXEC              */ {  216000,   330000 },
   /* FORM_C0                    */ {  917000,  1297000 },
   /* FORM_C1                    */ {  648000,   917000 },
   /* FORM_C2                    */ {  458000,   648000 },
   /* FORM_C3                    */ {  324000,   458000 },
   /* FORM_C4                    */ {  229000,   324000 },
   /* FORM_C5                    */ {  162000,   229000 },
   /* FORM_C6                    */ {  114000,   162000 },
   /* FORM_C6_C5                 */ {  114000,   229000 },
   /* FORM_C7                    */ {   81000,   114000 },
   /* FORM_C7_C6                 */ {   81000,   162000 },
   /* FORM_C8                    */ {   57000,    81000 },
   /* FORM_C9                    */ {   40000,    57000 },
   /* FORM_C10                   */ {   28000,    40000 },
   /* FORM_E1                    */ {  711200,  1016000 },
   /* FORM_RA0                   */ {  860000,  1220000 },
   /* FORM_RA1                   */ {  610000,   860000 },
   /* FORM_RA2                   */ {  430000,   610000 },
   /* FORM_SRA0                  */ {  900000,  1280000 },
   /* FORM_SRA1                  */ {  640000,   900000 },
   /* FORM_SRA2                  */ {  450000,   640000 },
   /* FORM_US_FANFOLD            */ {  377825,   279400 },
   /* FORM_FANFOLD_1             */ {  368300,   279400 },
   /* FORM_FANFOLD_2             */ {  304800,   215900 },
   /* FORM_FANFOLD_3             */ {  241300,   279400 },
   /* FORM_FANFOLD_4             */ {  215900,   304800 },
   /* FORM_FANFOLD_5             */ {  215900,   279400 },
   /* FORM_GERMAN_LEGAL_FANFOLD  */ {  215900,   330200 },
   /* FORM_EUROPEAN_FANFOLD      */ {  215900,   304800 },
   /* FORM_PANORAMIC             */ {  210000,   594000 },
   /* FORM_PHOTO_4_6             */ {  101600,   150000 },
   /* FORM_PHOTO_100_150         */ {  100000,   300000 },
   /* FORM_PHOTO_200_300         */ {  200000,   152400 },
   /* FORM_PLOTTER_SIZE_A        */ {  215900,   279400 },
   /* FORM_PLOTTER_SIZE_B        */ {  279400,   431800 },
   /* FORM_PLOTTER_SIZE_C        */ {  431800,   558800 },
   /* FORM_PLOTTER_SIZE_D        */ {  558800,   863600 },
   /* FORM_PLOTTER_SIZE_E        */ {  863690,  1117600 },
   /* FORM_PLOTTER_SIZE_F        */ { 1117600,  1727200 },
   /* FORM_US_A_ARCHITECTURAL    */ {  228600,   304800 },
   /* FORM_US_B_ARCHITECTURAL    */ {  304800,   457200 },
   /* FORM_US_C_ARCHITECTURAL    */ {  457200,   609600 },
   /* FORM_US_D_ARCHITECTURAL    */ {  609600,   914400 },
   /* FORM_US_E_ARCHITECTURAL    */ {  914400,  1219200 },
   /* FORM_SUPER_A3_B            */ {  328920,   483020 },
   /* FORM_SUPER_A               */ {  227076,   355600 },
   /* FORM_SUPER_B               */ {  330200,   482600 },
   /* FORM_ROLL_76_2MM           */ {   76200,        0 },
   /* FORM_ROLL_69_5MM           */ {   69500,        0 }
};

int DeviceForm::
sizeToID (int cx,
          int cy)
{
   for (int i = 0; i < (int)dimof (vaFormSizes); i++)
   {
      if (  cx == vaFormSizes[i].cx
         && cy == vaFormSizes[i].cy
         )
      {
         return i + 1;
      }
   }

   return DeviceForm::FORM_UNLISTED;
}

int DeviceForm::
getCx ()
{
   if (iXOverride_d)
   {
      return iXOverride_d;
   }

   if (  DeviceForm::FORM_USER_DEFINED > id_d
      || DeviceForm::FORM_ROLL_69_5MM  < id_d
      )
   {
      return 0;
   }

   return vaFormSizes[id_d - DeviceForm::FORM_USER_DEFINED].cx;
}

int DeviceForm::
getCy ()
{
   if (iYOverride_d)
   {
      return iYOverride_d;
   }

   if (  DeviceForm::FORM_USER_DEFINED > id_d
      || DeviceForm::FORM_ROLL_69_5MM  < id_d
      )
   {
      return 0;
   }

   return vaFormSizes[id_d - DeviceForm::FORM_USER_DEFINED].cy;
}

void DeviceForm::
setCx (int iXOverride)
{
   iXOverride_d = iXOverride;
}

void DeviceForm::
setCy (int iYOverride)
{
   iYOverride_d = iYOverride;
}

int DeviceForm::
getOverrideX ()
{
   return iXOverride_d;
}

int DeviceForm::
getOverrideY ()
{
   return iYOverride_d;
}

typedef struct _ReservedMap {
   const char *pszName;
   int         iValue;
} RESERVEDMAP, *PRESERVEDMAP;

const RESERVEDMAP aReservedKeywords[] = {
   { "NO_CAPABILITIES",             0x00000000 },
   { "FORM_CAPABILITY_ROLL",        0x80000000 },
   { "FORM_CAPABILITY_USERDEFINED", 0x40000000 }
};

bool DeviceForm::
isReservedKeyword (const char *pszId)
{
   for (int i = 0; i < (int)dimof (aReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, aReservedKeywords[i].pszName))
         return true;
   }

   return false;
}

int DeviceForm::
getReservedValue (const char *pszId)
{
   for (int i = 0; i < (int)dimof (aReservedKeywords); i++)
   {
      if (0 == strcmp (pszId, aReservedKeywords[i].pszName))
         return aReservedKeywords[i].iValue;
   }

   return 0;
}

string DeviceForm::
toString (ostrstream& oss)
{
   oss << hex
       << "{"
       << STRING_OF (id_d)
       << ", iCapabilities = 0x" << iCapabilities_d
       << ", hcInfo_d = " << *hcInfo_d
       << "}"
       << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceForm& const_self)
{
   DeviceForm& self = const_cast<DeviceForm&>(const_self);
   ostrstream oss;

   os << self.toString (oss);

   return os;
}
