/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceInstance.hpp"

#include <strstream>

DeviceInstance::
DeviceInstance (Device *pDevice)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::" << __FUNCTION__ << " ()" << endl;
#endif

   pDevice_d = pDevice;
}

DeviceInstance::
~DeviceInstance ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::~" << __FUNCTION__ << " ()" << endl;
#endif
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

void DeviceInstance::
initializeInstance ()
{
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

bool DeviceInstance::
handleDeviceJobProperty (char *pszKey,
                         char *pszValue)
{
   return false;
}

string * DeviceInstance::
queryDeviceJobProperties ()
{
   Enumeration *pEnum = this->listDeviceJobPropertyKeys ();

   // If the DeviceInstance class supports listDeviceJobPropertyKeys () then
   // enumerate the keys and return the values.
   if (pEnum)
   {
      ostrstream oss;
      bool       fSuccess = pEnum->hasMoreElements ();

      while (pEnum->hasMoreElements ())
      {
         char *pszKey = (char *)pEnum->nextElement ();

         if (!pszKey)
         {
#ifndef RETAIL
            if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "Error: Enumeration returns null!" << endl;
#endif
            fSuccess = false;
            break;
         }

         string *pstringValue = pDevice_d->getJobProperty (pszKey);

         if (!pstringValue)
         {
#ifndef RETAIL
            if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "Error: getJobProperty (" << pszKey << ") return null!" << endl;
#endif
            fSuccess = false;
            break;
         }

         oss << pszKey << "=" << *pstringValue;

         if (pEnum->hasMoreElements ())
            oss << " ";

         delete pstringValue;
      }

      delete pEnum;

      if (fSuccess)
      {
         oss << ends;

         return new string (oss.str ());
      }
   }

   return 0;
}

Enumeration * DeviceInstance::
listDeviceJobPropertyKeys ()
{
   class NullJobPropertyKeyEnumerator : public Enumeration
   {
   public:
      NullJobPropertyKeyEnumerator ()
      {
      }

      virtual bool hasMoreElements ()
      {
         return false;
      }

      virtual void * nextElement ()
      {
         return 0;
      }
   };

   return new NullJobPropertyKeyEnumerator ();
}

/*
 *  boolean                              "boolean true"
 *  integer with optional ranges         "integer 1 -5 42"
 *  floating point with optional ranges  "float 3.14157"
 *  string                               "string value"
 */
string * DeviceInstance::
getJobPropertyType (char *pszKey)
{
   return 0;
}

string * DeviceInstance::
getJobProperty (char *pszKey)
{
   return 0;
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

DeviceOrientation * DeviceInstance::
getCurrentOrientation ()
{
   return pDevice_d->getCurrentOrientation ();
}

char * DeviceInstance::
getCurrentDitherID ()
{
   return pDevice_d->getCurrentDitherID ();
}

DeviceForm * DeviceInstance::
getCurrentForm ()
{
   return pDevice_d->getCurrentForm ();
}

DeviceTray * DeviceInstance::
getCurrentTray ()
{
   return pDevice_d->getCurrentTray ();
}

DeviceMedia * DeviceInstance::
getCurrentMedia ()
{
   return pDevice_d->getCurrentMedia ();
}

DeviceResolution * DeviceInstance::
getCurrentResolution ()
{
   return pDevice_d->getCurrentResolution ();
}

DeviceCommand * DeviceInstance::
getCommands ()
{
   return pDevice_d->getCommands ();
}

DeviceData * DeviceInstance::
getDeviceData ()
{
   return pDevice_d->getDeviceData ();
}

DevicePrintMode * DeviceInstance::
getCurrentPrintMode ()
{
   return pDevice_d->getCurrentPrintMode ();
}

DeviceGamma * DeviceInstance::
getCurrentGamma ()
{
   return pDevice_d->getCurrentGamma ();
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

void DeviceInstance::
ditherNewFrame ()
{
   if (pDevice_d->getDeviceBlitter ())
      pDevice_d->getDeviceBlitter ()->ditherNewFrame ();
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

bool DeviceInstance::
hasCapability (long lMask)
{
   return pDevice_d->hasCapability (lMask);
}

bool DeviceInstance::
hasRasterCapability (long lMask)
{
   return pDevice_d->hasRasterCapability (lMask);
}

bool DeviceInstance::
hasDeviceOption (long lMask)
{
   return pDevice_d->hasDeviceOption (lMask);
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

bool DeviceInstance::
beginJob ()
{
   BinaryData *pCmd = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::beginJob ()" << endl;
#endif

   pCmd = getCommands ()->getCommandData ("cmdInit");

   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool DeviceInstance::
beginJob (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::beginJob (with props)" << endl;
#endif

   return beginJob ();
}

bool DeviceInstance::
newFrame ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::newFrame ()" << endl;
#endif

   return true;
}

bool DeviceInstance::
newFrame (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::newFrame (with props)" << endl;
#endif

   return newFrame ();
}

bool DeviceInstance::
endJob ()
{
   BinaryData *pCmd = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceInstance ()) cerr << "DeviceInstance::endJob ()" << endl;
#endif

   pCmd = getCommands ()->getCommandData ("cmdTerm");

   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool DeviceInstance::
abortJob ()
{
   return true;
}

/*8<--------8<--------8<--------8<--------8<--------8<--------8<--------*/

string DeviceInstance::
toString (ostrstream& oss)
{
   oss << hex << "{ pDevice_d = " << pDevice_d << " }" << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceInstance& const_self)
{
   DeviceInstance& self = const_cast<DeviceInstance&>(const_self);
   ostrstream      oss;

   os << self.toString (oss);

   return os;
}
