/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceMedia.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceMedia::
DeviceMedia (int         id,
             BinaryData *pbdData,
             int         iColorAdjustRequired,
             int         iAbsorption)
{
   id_d                   = id;
   pbdData_d              = pbdData;
   iColorAdjustRequired_d = iColorAdjustRequired;
   iAbsorption_d          = iAbsorption;
}

DeviceMedia::
~DeviceMedia ()
{
   delete pbdData_d;
}

// added by raharper@us.ibm.com for Foomatic
char * DeviceMedia::
getName (int id)
{
   return StringResource::getString (StringResource::STRINGGROUP_MEDIAS, id);
}

char * DeviceMedia::
getName ()
{
   return StringResource::getString (StringResource::STRINGGROUP_MEDIAS,
                                     id_d);
}

bool DeviceMedia::
isID (int id)
{
   return id == id_d;
}

int DeviceMedia::
getID ()
{
   return id_d;
}

BinaryData * DeviceMedia::
getData ()
{
   return pbdData_d;
}

int DeviceMedia::
getColorAdjustRequired ()
{
   return iColorAdjustRequired_d;
}

int DeviceMedia::
getAbsorption ()
{
   return iAbsorption_d;
}

DeviceMedia * DeviceMedia::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszMediaNames[x - MEDIA_UNLISTED])

static char *apszMediaNames[] = {
   "MEDIA_UNLISTED",
   "MEDIA_NONE",
   "MEDIA_AUTO",
   "MEDIA_PLAIN",
   "MEDIA_TRANSPARENCY",
   "MEDIA_GLOSSY",
   "MEDIA_SPECIAL",
   "MEDIA_COATED",
   "MEDIA_BACKPRINT",
   "MEDIA_CLOTH",
   "MEDIA_THICK",
   "MEDIA_HIGH_GLOSS_FILM",
   "MEDIA_HIGH_RESOLUTION",
   "MEDIA_SPECIAL_360",
   "MEDIA_SPECIAL_720",
   "MEDIA_PLAIN_ENHANCED",
   "MEDIA_IRON_ON",
   "MEDIA_LABECA",
   "MEDIA_THERMAL",
   "MEDIA_CD_MASTER",
   "MEDIA_CARDBOARD",
   "MEDIA_POSTCARD",
   "MEDIA_PHOTOGRAPHIC_PAPER",
   "MEDIA_PHOTOGRAPHIC_LABEL",
   "MEDIA_HP_PREMIUM_PAPER",
   "MEDIA_HP_PHOTOGRAPHIC_PAPER",
   "MEDIA_ENVELOPE",
   "MEDIA_PREPRINTED",
   "MEDIA_LETTERHEAD",
   "MEDIA_PREPUNCHED",
   "MEDIA_LABELS",
   "MEDIA_BOND",
   "MEDIA_RECYCLED",
   "MEDIA_COLOR",
   "MEDIA_CARDSTOCK",
   "MEDIA_ROUGH",
   "MEDIA_OTHER"
};

int DeviceMedia::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      MEDIA_AUTO,
      MEDIA_BACKPRINT,
      MEDIA_BOND,
      MEDIA_CARDBOARD,
      MEDIA_CARDSTOCK,
      MEDIA_CD_MASTER,
      MEDIA_CLOTH,
      MEDIA_COATED,
      MEDIA_COLOR,
      MEDIA_ENVELOPE,
      MEDIA_GLOSSY,
      MEDIA_HIGH_GLOSS_FILM,
      MEDIA_HIGH_RESOLUTION,
      MEDIA_HP_PHOTOGRAPHIC_PAPER,
      MEDIA_HP_PREMIUM_PAPER,
      MEDIA_IRON_ON,
      MEDIA_LABECA,
      MEDIA_LABELS,
      MEDIA_LETTERHEAD,
      MEDIA_NONE,
      MEDIA_OTHER,
      MEDIA_PHOTOGRAPHIC_LABEL,
      MEDIA_PHOTOGRAPHIC_PAPER,
      MEDIA_PLAIN,
      MEDIA_PLAIN_ENHANCED,
      MEDIA_POSTCARD,
      MEDIA_PREPRINTED,
      MEDIA_PREPUNCHED,
      MEDIA_RECYCLED,
      MEDIA_ROUGH,
      MEDIA_SPECIAL,
      MEDIA_SPECIAL_360,
      MEDIA_SPECIAL_720,
      MEDIA_THERMAL,
      MEDIA_THICK,
      MEDIA_TRANSPARENCY
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return MEDIA_UNLISTED;
}

char * DeviceMedia::
IDToName (int id)
{
   id -= MEDIA_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszMediaNames)
      )
      return apszMediaNames[id];
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + MEDIA_UNLISTED);

      return achUnknown;
   }
}

string DeviceMedia::
toString (ostrstream& oss)
{
   oss << dec
       << "{"
       << STRING_OF (id_d)
       << ", iColorAdjustRequired_d = " << iColorAdjustRequired_d
       << ", iAbsorption_d = " << iAbsorption_d
       << "}"
       << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceMedia& const_self)
{
   DeviceMedia& self = const_cast<DeviceMedia&>(const_self);
   ostrstream   oss;

   os << self.toString (oss);

   return os;
}
