/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceMedia
#define _DeviceMedia

#include "BinaryData.hpp"
#include "Enumeration.hpp"

#include <iostream>
#include <strstream>
#include <string>

class DeviceMedia
{
public:
   enum {
      MEDIA_UNLISTED              = -1,
      MEDIA_NONE,
      MEDIA_AUTO,                 /* aka default */
      MEDIA_PLAIN,
      MEDIA_TRANSPARENCY,
      MEDIA_GLOSSY,
      MEDIA_SPECIAL,
      MEDIA_COATED,
      MEDIA_BACKPRINT,
      MEDIA_CLOTH,
      MEDIA_THICK,
      MEDIA_HIGH_GLOSS_FILM,
      MEDIA_HIGH_RESOLUTION,
      MEDIA_SPECIAL_360,
      MEDIA_SPECIAL_720,
      MEDIA_PLAIN_ENHANCED,
      MEDIA_IRON_ON,
      MEDIA_LABECA,
      MEDIA_THERMAL,
      MEDIA_CD_MASTER,
      MEDIA_CARDBOARD,
      MEDIA_POSTCARD,
      MEDIA_PHOTOGRAPHIC_PAPER,
      MEDIA_PHOTOGRAPHIC_LABEL,
      MEDIA_HP_PREMIUM_PAPER,
      MEDIA_HP_PHOTOGRAPHIC_PAPER,
      MEDIA_ENVELOPE,
      MEDIA_PREPRINTED,
      MEDIA_LETTERHEAD,
      MEDIA_PREPUNCHED,
      MEDIA_LABELS,
      MEDIA_BOND,
      MEDIA_RECYCLED,
      MEDIA_COLOR,
      MEDIA_CARDSTOCK,
      MEDIA_ROUGH,
      MEDIA_OTHER
   };
   enum {
      COUNT = MEDIA_OTHER - MEDIA_UNLISTED + 1
   };

   enum {
      MEDIA_NO_ABSORPTION,
      MEDIA_LIGHT_ABSORPTION,
      MEDIA_HEAVY_ABSORPTION
   };

                        DeviceMedia            (int                id,
                                                BinaryData        *pbdData,
                                                int                iColorAdjustRequired,
                                                int                iAbsorption);
   virtual             ~DeviceMedia            ();

   // By bringing named creates into base class, it creates a lot of complexity
   static  DeviceMedia *create                 (int                id)    { return (DeviceMedia *)0; }
   DeviceMedia         *create                 (char              *pszId);
   static  int          nameToID               (char              *pszId);
   static  char        *IDToName               (int                id);
   // Added by raharper@us.ibm.com for Foomatic.cpp
   static  char        *getName                (int                id);

   char                *getName                ();
   bool                 isID                   (int                id);
   int                  getID                  ();
   int                  getTypeID              ();
   int                  getStringID            ();
   BinaryData          *getData                ();
   int                  getColorAdjustRequired ();
   int                  getAbsorption          ();
   // raharper@us.ibm.com  Adding method
   virtual bool         isMediaSupported       (int                id) = 0;

   virtual Enumeration *getEnumeration         () = 0;

   virtual string       toString               (ostrstream&        oss);
   friend ostream&      operator<<             (ostream&           os,
                                                const DeviceMedia& self);

protected:
   virtual DeviceMedia *createV                (int                id)    = 0;

   int         id_d;
   BinaryData *pbdData_d;
   int         iColorAdjustRequired_d;
   int         iAbsorption_d;
};

#endif
