/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceOrientation.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceOrientation::
DeviceOrientation (int id)
{
   id_d = id;
}

char * DeviceOrientation::
getName (int id)
{
   return StringResource::getString (StringResource::STRINGGROUP_ORIENTATIONS, id);
}

char * DeviceOrientation::
getName ()
{
   return StringResource::getString (StringResource::STRINGGROUP_ORIENTATIONS, id_d);
}

bool DeviceOrientation::
isID (int id)
{
   return id == id_d;
}

int DeviceOrientation::
getID ()
{
   return id_d;
}

DeviceOrientation * DeviceOrientation::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszOrientationNames[x - ORIENTATION_UNLISTED])

static char *apszOrientationNames[] = {
   "ORIENTATION_UNLISTED",
   "ORIENTATION_NONE",
   "ORIENTATION_PORTRAIT",
   "ORIENTATION_LANDSCAPE",
   "ORIENTATION_REVERSE_PORTRAIT",
   "ORIENTATION_REVERSE_LANDSCAPE"
};

int DeviceOrientation::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      ORIENTATION_LANDSCAPE,
      ORIENTATION_NONE,
      ORIENTATION_PORTRAIT,
      ORIENTATION_REVERSE_LANDSCAPE,
      ORIENTATION_REVERSE_PORTRAIT
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return ORIENTATION_UNLISTED;
}

char * DeviceOrientation::
IDToName (int id)
{
   id -= ORIENTATION_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszOrientationNames)
      )
   {
      return apszOrientationNames[id];
   }
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + ORIENTATION_PORTRAIT);

      return achUnknown;
   }
}

string DeviceOrientation::
toString (ostrstream& oss)
{
   oss << "{"
       << STRING_OF (id_d)
       << "}"
       << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceOrientation& const_self)
{
   DeviceOrientation& self = const_cast<DeviceOrientation&>(const_self);
   ostrstream         oss;

   os << self.toString (oss);

   return os;
}
