/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceOrientation
#define _DeviceOrientation

#include "BinaryData.hpp"
#include "Enumeration.hpp"

#include <iostream>
#include <strstream>
#include <string>

class DeviceOrientation {
public:
   enum {
      ORIENTATION_UNLISTED          = -1,
      ORIENTATION_NONE,
      ORIENTATION_PORTRAIT,
      ORIENTATION_LANDSCAPE,
      ORIENTATION_REVERSE_PORTRAIT,
      ORIENTATION_REVERSE_LANDSCAPE
   };
   enum {
      COUNT = ORIENTATION_REVERSE_LANDSCAPE - ORIENTATION_UNLISTED + 1
   };

                              DeviceOrientation     (int                      id);

   static  DeviceOrientation *create                (int                      id)    { return (DeviceOrientation *)0; }
   DeviceOrientation         *create                (char                    *pszId);
   static  int               nameToID               (char                    *pszId);
   static  char             *IDToName               (int                      id);

   static char              *getName                (int                      id);

   char                     *getName                ();
   bool                      isID                   (int                      id);
   int                       getID                  ();

   virtual bool              isOrientationSupported (int                      id) = 0;

   virtual Enumeration      *getEnumeration         () = 0;

   virtual string            toString               (ostrstream&              oss);
   friend ostream&           operator<<             (ostream&                 os,
                                                     const DeviceOrientation& self);

protected:
   virtual DeviceOrientation *createV               (int                      id) = 0;

   int         id_d;
};

#endif
