/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "DeviceResolution.hpp"
#include "StringResource.hpp"
#include <stdio.h>
#include <string.h>

DeviceResolution::
DeviceResolution (int         id,
                  int         iXRes,
                  int         iYRes,
                  int         iXInternalRes,
                  int         iYInternalRes,
                  BinaryData *pbdData,
                  int         iCapabilities,
                  int         iDestinationBitsPerPel,
                  int         iScanlineMultiple)
{
   id_d                     = id;
   iXRes_d                  = iXRes;
   iYRes_d                  = iYRes;
   iXInternalRes_d          = iXInternalRes;
   iYInternalRes_d          = iYInternalRes;
   pbdData_d                = pbdData;
   iCapabilities_d          = iCapabilities;
   iDestinationBitsPerPel_d = iDestinationBitsPerPel;
   iScanlineMultiple_d      = iScanlineMultiple;
}

DeviceResolution::
~DeviceResolution ()
{
   delete pbdData_d;
}

char * DeviceResolution::
getName (int id)
{
   return StringResource::getString (StringResource::STRINGGROUP_RESOLUTIONS, id);
}

char * DeviceResolution::
getName ()
{
   return StringResource::getString (StringResource::STRINGGROUP_RESOLUTIONS, id_d);
}

bool DeviceResolution::
isID (int id)
{
   return id == id_d;
}

int DeviceResolution::
getID ()
{
   return id_d;
}

int DeviceResolution::
getXRes ()
{
   if (iXInternalRes_d > 0)
   {
      return iXInternalRes_d;
   }
   else
   {
      return iXRes_d;
   }
}

int DeviceResolution::
getYRes ()
{
   if (iYInternalRes_d > 0)
   {
      return iYInternalRes_d;
   }
   else
   {
      return iYRes_d;
   }
}

int DeviceResolution::
getExternalXRes ()
{
   return iXRes_d;
}

int DeviceResolution::
getExternalYRes ()
{
   return iYRes_d;
}

BinaryData * DeviceResolution::
getData ()
{
   return pbdData_d;
}

bool DeviceResolution::
hasCapability (int iFlag)
{
   return 0 != (iFlag & iCapabilities_d);
}

int DeviceResolution::
getCapabilities ()
{
   return iCapabilities_d;
}

int DeviceResolution::
getDstBitsPerPel ()
{
   return iDestinationBitsPerPel_d;
}

int DeviceResolution::
getScanlineMultiple ()
{
   return iScanlineMultiple_d;
}

void DeviceResolution::
setInternalXRes (int iXInternalRes)
{
   iXInternalRes_d = iXInternalRes;
}

void DeviceResolution::
setInternalYRes (int iYInternalRes)
{
   iYInternalRes_d = iYInternalRes;
}

DeviceResolution * DeviceResolution::
create (char *pszId)
{
   return this->createV (nameToID (pszId));
}

#define STRING_OF(x) (apszResolutionNames[x - RESOLUTION_UNLISTED])

static char *apszResolutionNames[] = {
   "RESOLUTION_UNLISTED",
   "RESOLUTION_NONE",
   "RESOLUTION_60_X_60",
   "RESOLUTION_60_X_72",
   "RESOLUTION_60_X_180",
   "RESOLUTION_60_X_360",
   "RESOLUTION_72_X_72",
   "RESOLUTION_75_X_75",
   "RESOLUTION_80_X_60",
   "RESOLUTION_80_X_72",
   "RESOLUTION_90_X_180",
   "RESOLUTION_90_X_360",
   "RESOLUTION_90_X_60",
   "RESOLUTION_90_X_72",
   "RESOLUTION_90_X_90",
   "RESOLUTION_100_X_100",
   "RESOLUTION_120_X_60",
   "RESOLUTION_120_X_72",
   "RESOLUTION_120_X_144",
   "RESOLUTION_120_X_180",
   "RESOLUTION_120_X_360",
   "RESOLUTION_144_X_72",
   "RESOLUTION_150_X_150",
   "RESOLUTION_180_X_180",
   "RESOLUTION_180_X_360",
   "RESOLUTION_200_X_200",
   "RESOLUTION_240_X_60",
   "RESOLUTION_240_X_72",
   "RESOLUTION_240_X_144",
   "RESOLUTION_240_X_240",
   "RESOLUTION_300_X_300",
   "RESOLUTION_360_X_180",
   "RESOLUTION_360_X_360",
   "RESOLUTION_360_X_720",
   "RESOLUTION_400_X_400",
   "RESOLUTION_600_X_300",
   "RESOLUTION_600_X_600",
   "RESOLUTION_720_X_360",
   "RESOLUTION_720_X_720",
   "RESOLUTION_1200_X_600",
   "RESOLUTION_1200_X_1200",
   "RESOLUTION_1440_X_720",
   "RESOLUTION_2880_X_720",
   "RESOLUTION_DRAFT",
   "RESOLUTION_FINE",
   "RESOLUTION_GROUP_3",
   "RESOLUTION_GROUP_4",
   "RESOLUTION_HIGH",
   "RESOLUTION_LOW",
   "RESOLUTION_MEDIUM",
   "RESOLUTION_NORMAL",
   "RESOLUTION_PHOTO_QUALITY",
   "RESOLUTION_PRESENTATION"
};

int DeviceResolution::
nameToID (char *pszId)
{
   static short int asiEntries[] = {
      RESOLUTION_100_X_100,
      RESOLUTION_1200_X_1200,
      RESOLUTION_1200_X_600,
      RESOLUTION_120_X_144,
      RESOLUTION_120_X_180,
      RESOLUTION_120_X_360,
      RESOLUTION_120_X_60,
      RESOLUTION_120_X_72,
      RESOLUTION_1440_X_720,
      RESOLUTION_144_X_72,
      RESOLUTION_150_X_150,
      RESOLUTION_180_X_180,
      RESOLUTION_180_X_360,
      RESOLUTION_200_X_200,
      RESOLUTION_240_X_144,
      RESOLUTION_240_X_240,
      RESOLUTION_240_X_60,
      RESOLUTION_240_X_72,
      RESOLUTION_2880_X_720,
      RESOLUTION_300_X_300,
      RESOLUTION_360_X_180,
      RESOLUTION_360_X_360,
      RESOLUTION_360_X_720,
      RESOLUTION_400_X_400,
      RESOLUTION_600_X_300,
      RESOLUTION_600_X_600,
      RESOLUTION_60_X_180,
      RESOLUTION_60_X_360,
      RESOLUTION_60_X_60,
      RESOLUTION_60_X_72,
      RESOLUTION_720_X_360,
      RESOLUTION_720_X_720,
      RESOLUTION_72_X_72,
      RESOLUTION_75_X_75,
      RESOLUTION_80_X_60,
      RESOLUTION_80_X_72,
      RESOLUTION_90_X_180,
      RESOLUTION_90_X_360,
      RESOLUTION_90_X_60,
      RESOLUTION_90_X_72,
      RESOLUTION_90_X_90,
      RESOLUTION_DRAFT,
      RESOLUTION_FINE,
      RESOLUTION_GROUP_3,
      RESOLUTION_GROUP_4,
      RESOLUTION_HIGH,
      RESOLUTION_LOW,
      RESOLUTION_MEDIUM,
      RESOLUTION_NONE,
      RESOLUTION_NORMAL,
      RESOLUTION_PHOTO_QUALITY,
      RESOLUTION_PRESENTATION
   };

   int iLow  = 0;
   int iMid  = (int)dimof (asiEntries) / 2;
   int iHigh = (int)dimof (asiEntries) - 1;
   int iResult;

   while (iLow <= iHigh)
   {
      iResult = strcmp (pszId, STRING_OF (asiEntries[iMid]));

      if (0 == iResult)
      {
         return asiEntries[iMid];
      }
      else if (0 > iResult)
      {
         // str1 < str2
         iHigh = iMid - 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
      else // (0 < iResult)
      {
         // str1 > str2
         iLow  = iMid + 1;
         iMid  = iLow + (iHigh - iLow) / 2;
      }
   }

   return RESOLUTION_UNLISTED;
}

char * DeviceResolution::
IDToName (int id)
{
   id -= RESOLUTION_UNLISTED;

   if (  0 <= id
      && id < (int)dimof (apszResolutionNames)
      )
      return apszResolutionNames[id];
   else
   {
      static char achUnknown[21];

      sprintf (achUnknown, "Unknown (%d)", id + RESOLUTION_UNLISTED);

      return achUnknown;
   }
}

string DeviceResolution::
toString (ostrstream& oss)
{
   oss << dec
       << "{"
       << STRING_OF (id_d)
       << ", iXRes_d = " << iXRes_d
       << ", iYRes_d = " << iYRes_d
       << hex << ", iCapabilities_d = 0x" << iCapabilities_d
       << dec << ", iDestinationBitsPerPel_d = " << iDestinationBitsPerPel_d
       << ", iScanlineMultiple_d = " << iScanlineMultiple_d
       << "}"
       << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const DeviceResolution& const_self)
{
   DeviceResolution& self = const_cast<DeviceResolution&>(const_self);
   ostrstream        oss;

   os << self.toString (oss);

   return os;
}
