/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _DeviceResolution
#define _DeviceResolution

#include "BinaryData.hpp"
#include "Enumeration.hpp"

#include <iostream>
#include <strstream>
#include <string>

class DeviceResolution
{
public:
   enum {
      RESOLUTION_UNLISTED                = -1,
      RESOLUTION_NONE,
      RESOLUTION_60_X_60,
      RESOLUTION_60_X_72,
      RESOLUTION_60_X_180,
      RESOLUTION_60_X_360,
      RESOLUTION_72_X_72,
      RESOLUTION_75_X_75,
      RESOLUTION_80_X_60,
      RESOLUTION_80_X_72,
      RESOLUTION_90_X_180,
      RESOLUTION_90_X_360,
      RESOLUTION_90_X_60,
      RESOLUTION_90_X_72,
      RESOLUTION_90_X_90,
      RESOLUTION_100_X_100,
      RESOLUTION_120_X_60,
      RESOLUTION_120_X_72,
      RESOLUTION_120_X_144,
      RESOLUTION_120_X_180,
      RESOLUTION_120_X_360,
      RESOLUTION_144_X_72,
      RESOLUTION_150_X_150,
      RESOLUTION_180_X_180,
      RESOLUTION_180_X_360,
      RESOLUTION_200_X_200,
      RESOLUTION_240_X_60,
      RESOLUTION_240_X_72,
      RESOLUTION_240_X_144,
      RESOLUTION_240_X_240,
      RESOLUTION_300_X_300,
      RESOLUTION_360_X_180,
      RESOLUTION_360_X_360,
      RESOLUTION_360_X_720,
      RESOLUTION_400_X_400,
      RESOLUTION_600_X_300,
      RESOLUTION_600_X_600,
      RESOLUTION_720_X_360,
      RESOLUTION_720_X_720,
      RESOLUTION_1200_X_600,
      RESOLUTION_1200_X_1200,
      RESOLUTION_1440_X_720,
      RESOLUTION_2880_X_720,
      RESOLUTION_DRAFT,
      RESOLUTION_FINE,
      RESOLUTION_GROUP_3,
      RESOLUTION_GROUP_4,
      RESOLUTION_HIGH,
      RESOLUTION_LOW,
      RESOLUTION_MEDIUM,
      RESOLUTION_NORMAL,
      RESOLUTION_PHOTO_QUALITY,
      RESOLUTION_PRESENTATION
   };
   enum {
      COUNT = RESOLUTION_PRESENTATION - RESOLUTION_UNLISTED + 1
   };

                             DeviceResolution      (int                     id,
                                                    int                     iXRes,
                                                    int                     iYRes,
                                                    int                     iXInternalRes,
                                                    int                     iYInternalRes,
                                                    BinaryData             *pbdData,
                                                    int                     iCapabilities,
                                                    int                     iDestinationBitsPerPel,
                                                    int                     iScanlineMultiple);
   virtual                  ~DeviceResolution      ();

   static DeviceResolution  *create                (int                     id)    { return (DeviceResolution *)0; }
   DeviceResolution         *create                (char                   *pszId);
   static int                nameToID              (char                   *pszId);
   static char              *IDToName              (int                     id);

   static char              *getName               (int                     id);

   char                     *getName               ();
   bool                      isID                  (int                     id);
   int                       getID                 ();
   int                       getXRes               ();
   int                       getYRes               ();
   int                       getExternalXRes       ();
   int                       getExternalYRes       ();
   BinaryData               *getData               ();
   bool                      hasCapability         (int                     iFlag);
   int                       getCapabilities       ();
   int                       getDstBitsPerPel      ();
   int                       getScanlineMultiple   ();

   void                      setInternalXRes       (int                     iXInternalRes);
   void                      setInternalYRes       (int                     iYInternalRes);

   virtual bool              isResolutionSupported (int                     id) = 0;

   virtual Enumeration      *getEnumeration        () = 0;

   virtual string            toString              (ostrstream&             oss);
   friend ostream&           operator<<            (ostream&                os,
                                                    const DeviceResolution& self);

protected:
   virtual DeviceResolution *createV               (int                     id) = 0;

   int         id_d;
   int         iXRes_d;
   int         iYRes_d;
   int         iXInternalRes_d;
   int         iYInternalRes_d;
   BinaryData *pbdData_d;
   int         iCapabilities_d;
   int         iDestinationBitsPerPel_d;
   int         iScanlineMultiple_d;
};

#endif
