/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"

#include <stdio.h>
#include <string.h>

#include <glib.h>
#include <gmodule.h>

GModule *hmodDevice;

int
main (int argc, char *argv[])
{
   char          achLibDeviceName[512];
   Device       *pDevice               = 0;
   PFNNEWDEVICE  pfnNewDevice          = 0;

   if (!g_module_supported ())
   {
      cerr << "This program needs glib's module routines!" << endl;
      return 1;
   }

   if (2 != argc)
   {
      cerr << "Usage: " << argv[0] << " printer_library_name" << endl;
      return 1;
   }

   if (0 != strncasecmp (argv[1], "lib", 3))
   {
      sprintf (achLibDeviceName, "lib%s.so", argv[1]);
   }
   else
   {
      strcpy (achLibDeviceName, argv[1]);
   }

   hmodDevice = g_module_open (achLibDeviceName, (GModuleFlags)0);
   if (!hmodDevice)
   {
      cerr << "g_module_error returns " << g_module_error () << endl;
      return 2;
   }

   g_module_symbol (hmodDevice, "newDevice__Fb", (gpointer *)&pfnNewDevice);

   if (!pfnNewDevice)
      return 3;

   pDevice = pfnNewDevice (true);

   cout << pDevice->getDriverName () << "." << pDevice->getDeviceName () << endl;

   delete pDevice;

   int rc = 0;

   rc = g_module_close (hmodDevice);

   return 0;
}
