/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "Device.hpp"
#include "Omni.hpp"

#include <glib.h>
#include <gmodule.h>

#if 1

int
main (int argc, char *argv[])
{
   if (!g_module_supported ())
   {
      cerr << "This program needs glib's module routines!" << endl;
      return 1;
   }

   bool         fDescriptive = false;
   Enumeration *pEnum        = Omni::listDevices ();

   for (int i = 1; i < argc; i++)
   {
      if (0 == strcasecmp (argv[i], "descriptive"))
      {
         fDescriptive = true;
      }
   }

   while (pEnum->hasMoreElements ())
   {
      char *pszLibName = (char *)pEnum->nextElement ();

      if (fDescriptive)
      {
         Device       *pDevice      = 0;
         PFNNEWDEVICE  pfnNewDevice = 0;
         GModule      *hmodDevice   = 0;

         hmodDevice = g_module_open (pszLibName, (GModuleFlags)0);

         if (hmodDevice)
         {
            g_module_symbol (hmodDevice, "newDevice__Fb", (gpointer *)&pfnNewDevice);

            if (pfnNewDevice)
            {
               pDevice = pfnNewDevice (true);

               cerr << pDevice->getDriverName () << "." << pDevice->getDeviceName () << endl;

               delete pDevice;
            }

            g_module_close (hmodDevice);
         }
      }
      else
      {
         cerr << pszLibName << endl;
      }
   }

   delete pEnum;

   return 0;
}

#else

int
main (int argc, char *argv[])
{
   Device *pDevice = 0;
   void   *hDevice = 0;

   pDevice = Omni::createDevice ("Epson",
                                 "Epson LQ-2550",
                                 0,
                                 false,
                                 &hDevice);

   if (pDevice)
   {
      cerr << "Success! " << *pDevice << endl;
   }

   // Clean up!
   delete pDevice;

   if (hDevice)
   {
      dlclose (hDevice);
   }

   return 0;
}

#endif
