/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include <fstream.h>

#include "Device.hpp"
#include "defines.hpp"

#include <glib.h>
#include <gmodule.h>

GModule *vhmodDevice = 0;

typedef unsigned int ULONG;
typedef short int SHORT;
typedef unsigned short int USHORT;

/* bitmap parameterization used by GpiCreateBitmap and others */
typedef struct _BITMAPINFOHEADER         /* bmp */
{
   ULONG  cbFix;
   USHORT cx;
   USHORT cy;
   USHORT cPlanes;
   USHORT cBitCount;
} BITMAPINFOHEADER;
typedef BITMAPINFOHEADER *PBITMAPINFOHEADER;

///* RGB data for _BITMAPINFO struct */
//typedef struct _RGB              /* rgb */
//{
//   BYTE bBlue;
//   BYTE bGreen;
//   BYTE bRed;
//} RGB;

/* bitmap data used by GpiSetBitmapBits and others */
typedef struct _BITMAPINFO       /* bmi */
{
   ULONG  cbFix;
   USHORT cx;
   USHORT cy;
   USHORT cPlanes;
   USHORT cBitCount;
   RGB    argbColor[1];
} BITMAPINFO;
typedef BITMAPINFO *PBITMAPINFO;

/* Constants for compression/decompression command */

#define CBD_COMPRESSION     1L
#define CBD_DECOMPRESSION   2L
#define CBD_BITS            0L

/* Flags for compression/decompression option */

#define CBD_COLOR_CONVERSION    0x00000001L

/* Compression scheme in the ulCompression field of the bitmapinfo structure */

#define BCA_UNCOMP          0L
#define BCA_HUFFMAN1D       3L
#define BCA_RLE4            2L
#define BCA_RLE8            1L
#define BCA_RLE24           4L

#define BRU_METRIC          0L

#define BRA_BOTTOMUP        0L

#define BRH_NOTHALFTONED    0L
#define BRH_ERRORDIFFUSION  1L
#define BRH_PANDA           2L
#define BRH_SUPERCIRCLE     3L

#define BCE_PALETTE         (-1L)
#define BCE_RGB             0L

typedef struct _BITMAPINFOHEADER2        /* bmp2  */
{
   ULONG  cbFix;            /* Length of structure                    */
   ULONG  cx;               /* Bit-map width in pels                  */
   ULONG  cy;               /* Bit-map height in pels                 */
   USHORT cPlanes;          /* Number of bit planes                   */
   USHORT cBitCount;        /* Number of bits per pel within a plane  */
   ULONG  ulCompression;    /* Compression scheme used to store the bitmap */
   ULONG  cbImage;          /* Length of bit-map storage data in bytes*/
   ULONG  cxResolution;     /* x resolution of target device          */
   ULONG  cyResolution;     /* y resolution of target device          */
   ULONG  cclrUsed;         /* Number of color indices used           */
   ULONG  cclrImportant;    /* Number of important color indices      */
   USHORT usUnits;          /* Units of measure                       */
   USHORT usReserved;       /* Reserved                               */
   USHORT usRecording;      /* Recording algorithm                    */
   USHORT usRendering;      /* Halftoning algorithm                   */
   ULONG  cSize1;           /* Size value 1                           */
   ULONG  cSize2;           /* Size value 2                           */
   ULONG  ulColorEncoding;  /* Color encoding                         */
   ULONG  ulIdentifier;     /* Reserved for application use           */
} BITMAPINFOHEADER2;
typedef BITMAPINFOHEADER2 *PBITMAPINFOHEADER2;

//typedef struct _RGB2         /* rgb2 */
//{
//   BYTE bBlue;              /* Blue component of the color definition */
//   BYTE bGreen;             /* Green component of the color definition*/
//   BYTE bRed;               /* Red component of the color definition  */
//   BYTE fcOptions;          /* Reserved, must be zero                 */
//} RGB2;
//typedef RGB2 *PRGB2;

//typedef struct _BITMAPINFO2      /* bmi2 */
//{
//   ULONG  cbFix;            /* Length of fixed portion of structure   */
//   ULONG  cx;               /* Bit-map width in pels                  */
//   ULONG  cy;               /* Bit-map height in pels                 */
//   USHORT cPlanes;          /* Number of bit planes                   */
//   USHORT cBitCount;        /* Number of bits per pel within a plane  */
//   ULONG  ulCompression;    /* Compression scheme used to store the bitmap */
//   ULONG  cbImage;          /* Length of bit-map storage data in bytes*/
//   ULONG  cxResolution;     /* x resolution of target device          */
//   ULONG  cyResolution;     /* y resolution of target device          */
//   ULONG  cclrUsed;         /* Number of color indices used           */
//   ULONG  cclrImportant;    /* Number of important color indices      */
//   USHORT usUnits;          /* Units of measure                       */
//   USHORT usReserved;       /* Reserved                               */
//   USHORT usRecording;      /* Recording algorithm                    */
//   USHORT usRendering;      /* Halftoning algorithm                   */
//   ULONG  cSize1;           /* Size value 1                           */
//   ULONG  cSize2;           /* Size value 2                           */
//   ULONG  ulColorEncoding;  /* Color encoding                         */
//   ULONG  ulIdentifier;     /* Reserved for application use           */
//   RGB2   argbColor[1];     /* Color definition record                */
//} BITMAPINFO2;
//typedef BITMAPINFO2 *PBITMAPINFO2;

typedef struct _BITMAPFILEHEADER    /* bfh */
{
   USHORT    usType;
   ULONG     cbSize;
   SHORT     xHotspot;
   SHORT     yHotspot;
   ULONG     offBits;
   BITMAPINFOHEADER bmp;
} BITMAPFILEHEADER;
typedef BITMAPFILEHEADER *PBITMAPFILEHEADER;

typedef struct _BITMAPARRAYFILEHEADER       /* bafh */
{
   USHORT    usType;
   ULONG     cbSize;
   ULONG     offNext;
   USHORT    cxDisplay;
   USHORT    cyDisplay;
   BITMAPFILEHEADER bfh;
} BITMAPARRAYFILEHEADER;
typedef BITMAPARRAYFILEHEADER *PBITMAPARRAYFILEHEADER;

typedef struct _BITMAPFILEHEADER2    /* bfh2 */
{
   USHORT    usType;
   ULONG     cbSize;
   SHORT     xHotspot;
   SHORT     yHotspot;
   ULONG     offBits;
   BITMAPINFOHEADER2 bmp2;
} BITMAPFILEHEADER2;
typedef BITMAPFILEHEADER2 *PBITMAPFILEHEADER2;

typedef struct _BITMAPARRAYFILEHEADER2    /* bafh2 */
{
   USHORT    usType;
   ULONG     cbSize;
   ULONG     offNext;
   USHORT    cxDisplay;
   USHORT    cyDisplay;
   BITMAPFILEHEADER2 bfh2;
} BITMAPARRAYFILEHEADER2;
typedef BITMAPARRAYFILEHEADER2 *PBITMAPARRAYFILEHEADER2;

/*************************************************************************
* These are the identifying values that go in the usType field of the
* BITMAPFILEHEADER(2) and BITMAPARRAYFILEHEADER(2).
* (BFT_ => Bit map File Type)
*************************************************************************/
#define BFT_ICON           0x4349   /* 'IC' */
#define BFT_BMAP           0x4d42   /* 'BM' */
#define BFT_POINTER        0x5450   /* 'PT' */
#define BFT_COLORICON      0x4943   /* 'CI' */
#define BFT_COLORPOINTER   0x5043   /* 'CP' */
#define BFT_BITMAPARRAY    0x4142   /* 'BA' */

void
bitblt (FILE   *fp,
        USHORT  usType,
        int     iMaxScanLines,
        Device *pDevice)
{
   PBYTE              pbBitmapBits      = 0;
   PBITMAPINFO2       pbmi2Bitmap       = 0;
   RECTL              rectlPageLocation = {0, 0, 0, 0};
   ULONG              cbFix;
   BITMAPFILEHEADER   bfhBitmap;
   BITMAPINFOHEADER2  bmih2Bitmap;
   int                iNumColorsBitmap;
   int                iBytesToAlloc;
   int                icbBitmapScanline;
   int                iScanlineMultiple;
   int                iNumBands;
   int                iCurrentBand;
   int                rc;

   if (!fp)
      return;

   bfhBitmap.usType = usType;

   rc = fread (&bfhBitmap.cbSize,
               ( sizeof (bfhBitmap.cbSize)
               + sizeof (bfhBitmap.xHotspot)
               + sizeof (bfhBitmap.yHotspot)
               + sizeof (bfhBitmap.offBits)
               ),
               1,
               fp);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << hex << "DeviceTester6: bfhBitmap.usType   = 0x" << bfhBitmap.usType << endl;
      cerr << "DeviceTester6: bfhBitmap.cbSize   = 0x" << bfhBitmap.cbSize << endl;
      cerr << "DeviceTester6: bfhBitmap.xHotspot = 0x" << bfhBitmap.xHotspot << endl;
      cerr << "DeviceTester6: bfhBitmap.yHotspot = 0x" << bfhBitmap.yHotspot << endl;
      cerr << "DeviceTester6: bfhBitmap.offBits  = 0x" << bfhBitmap.offBits << endl;
   }
#endif

   rc = fread (&cbFix, sizeof (cbFix), 1, fp);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << dec << "DeviceTester6: sizeof (BITMAPINFOHEADER) = " << sizeof (BITMAPINFOHEADER) << endl;
#endif

   if (sizeof (BITMAPINFOHEADER) >= cbFix)
   {
      // The format is BITMAPINFOHEAHER
      // @TBD
   }
   else
   {
      // The format is BITMAPINFOHEADER2
#ifndef RETAIL
      if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << hex << "DeviceTester6:    cbFix           = 0x" << cbFix << endl;
#endif
      bmih2Bitmap.cbFix = cbFix;
      rc = fread (&bmih2Bitmap.cx, bmih2Bitmap.cbFix - sizeof (bmih2Bitmap.cbFix), 1, fp);

#ifndef RETAIL
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cy))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << dec << "DeviceTester6:    cx              = " << bmih2Bitmap.cx << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cPlanes))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cy              = " << bmih2Bitmap.cy << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cBitCount))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cPlanes         = " << bmih2Bitmap.cPlanes << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, ulCompression))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cBitCount       = " << bmih2Bitmap.cBitCount << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cbImage))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    ulCompression   = " << bmih2Bitmap.ulCompression << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cxResolution))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cbImage         = " << bmih2Bitmap.cbImage << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cyResolution))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cxResolution    = " << bmih2Bitmap.cxResolution << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cclrUsed))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cyResolution    = " << bmih2Bitmap.cyResolution << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cclrImportant))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cclrUsed        = " << bmih2Bitmap.cclrUsed << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, usUnits))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cclrImportant   = " << bmih2Bitmap.cclrImportant << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, usReserved))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    usUnits         = " << bmih2Bitmap.usUnits << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, usRecording))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    usReserved      = " << bmih2Bitmap.usReserved << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cSize1))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    usRendering     = " << bmih2Bitmap.usRendering << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, cSize2))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cSize1          = " << bmih2Bitmap.cSize1 << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, ulColorEncoding))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    cSize2          = " << bmih2Bitmap.cSize2 << endl;
      if (bmih2Bitmap.cbFix >= FIELDOFFSET (BITMAPINFOHEADER2, ulIdentifier))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    ulColorEncoding = " << bmih2Bitmap.ulColorEncoding << endl;
      if (bmih2Bitmap.cbFix >= sizeof (BITMAPINFOHEADER2))
         if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6:    ulIdentifier    = " << bmih2Bitmap.ulIdentifier << endl;
#endif

      if (  !bmih2Bitmap.cclrUsed
         && 8 >= bmih2Bitmap.cBitCount
         )
      {
         // Fix the number of colors specified in the color table
         bmih2Bitmap.cclrUsed      = 1 << bmih2Bitmap.cBitCount;
         bmih2Bitmap.cclrImportant = 0;                    // All are important
      }
   }

   iNumColorsBitmap = 1 << bmih2Bitmap.cBitCount;

   // Allocate a full bitmap info 2 and bitmap header message
   iBytesToAlloc = sizeof (BITMAPINFO2);
   if (256 >= iNumColorsBitmap)
      iBytesToAlloc += (iNumColorsBitmap - 1) * sizeof (RGB2);

   pbmi2Bitmap = (PBITMAPINFO2)calloc (1, iBytesToAlloc);
   if (!pbmi2Bitmap)
      goto done;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << dec << "DeviceTester6: malloced " << iBytesToAlloc << " bytes at 0x" << hex << (int)pbmi2Bitmap << " for pbmi2Bitmap" << endl;
#endif

   // Initialize it
   pbmi2Bitmap->cbFix         = sizeof (BITMAPINFO2)
                              - sizeof (((PBITMAPINFO2)0)->argbColor);
   pbmi2Bitmap->cx            = bmih2Bitmap.cx;
   pbmi2Bitmap->cy            = bmih2Bitmap.cy;
   pbmi2Bitmap->cPlanes       = bmih2Bitmap.cPlanes;
   pbmi2Bitmap->cBitCount     = bmih2Bitmap.cBitCount;
   pbmi2Bitmap->cclrUsed      = 1 << bmih2Bitmap.cBitCount;
   pbmi2Bitmap->cclrImportant = 0;                            // All are important

   // Read the color table, if there is one
   if (256 >= iNumColorsBitmap)
   {
      if (sizeof (BITMAPINFOHEADER) >= cbFix)
      {
         /* We have to do this the hard way and read in the old RGB on top
         ** of the new RGB2
         */
         for (int i = 0; i < iNumColorsBitmap; i++)
         {
            rc = fread (&pbmi2Bitmap->argbColor[i], sizeof (RGB), 1, fp);
         }
      }
      else
      {
         rc = fread (pbmi2Bitmap->argbColor, iNumColorsBitmap * sizeof (RGB2), 1, fp);
      }
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << dec << "DeviceTester6: pbmi2Bitmap->= { "
           << "cx = " << pbmi2Bitmap->cx
           << ", cy = " << pbmi2Bitmap->cy
           << ", cPlanes = " << pbmi2Bitmap->cPlanes
           << ", cBitCount = " << pbmi2Bitmap->cBitCount
           << ", cclrUsed = " << pbmi2Bitmap->cclrUsed
           << ", cclrImportant = " << pbmi2Bitmap->cclrImportant
           << " }" << endl;
   }
#endif

   // Scan lines are double word aligned (32-bits)
   icbBitmapScanline  = ((pbmi2Bitmap->cx   * pbmi2Bitmap->cBitCount   + 31) >> 5) << 2;
   iBytesToAlloc      = icbBitmapScanline * iMaxScanLines;

   /* Bump up the # of scanlines that we send to the device to be a
   ** multiple of what it requests.
   */
   iScanlineMultiple = pDevice->getScanlineMultiple ();
   if (0 != (iMaxScanLines % iScanlineMultiple))
   {
      iMaxScanLines += iScanlineMultiple - (iMaxScanLines % iScanlineMultiple);
   }
   iNumBands = (pbmi2Bitmap->cy + iMaxScanLines - 1) / iMaxScanLines;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << dec << "DeviceTester6: iBytesToAlloc = " << iBytesToAlloc <<  endl;
      cerr        << "DeviceTester6: icbBitmapScanline = " << icbBitmapScanline <<  endl;
      cerr        << "DeviceTester6: iMaxScanLines = " << iMaxScanLines << endl;
      cerr        << "DeviceTester6: iNumBands = " << iNumBands << endl;
      cerr        << "DeviceTester6: iScanlineMultiple = " << iScanlineMultiple << endl;
   }
#endif

   pbBitmapBits = (PBYTE)calloc (1, iBytesToAlloc);
   if (!pbBitmapBits)
      goto done;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << "DeviceTester6: malloced " << iBytesToAlloc << " bytes at 0x" << hex << (int)pbBitmapBits << " for pbBitmapBits" << endl;
#endif

   // loop through the bands
   for (iCurrentBand = 0; iCurrentBand < iNumBands; iCurrentBand++)
   {
      // @TBD which way is it orientated?
      rectlPageLocation.xLeft   = 0;
      rectlPageLocation.xRight  = pbmi2Bitmap->cx - 1;
      rectlPageLocation.yTop    = (pbmi2Bitmap->cy - 1) - iCurrentBand * iMaxScanLines;
      rectlPageLocation.yBottom = rectlPageLocation.yTop - iMaxScanLines + 1;

      if (0 > rectlPageLocation.yBottom)
         rectlPageLocation.yBottom = 0;

#ifndef RETAIL
      if (DebugOutput::shouldOutputDeviceTester6 ())
         cerr << hex << "DeviceTester6: Bitblt: pbBitmapBits = 0x" << (int)pbBitmapBits
                     << ", bmi2Printer = 0x" << (int)pbmi2Bitmap
              << dec << ", rectlPageLocation = { (" << rectlPageLocation.xLeft
                     << ", " << rectlPageLocation.yBottom
                     << "), (" << rectlPageLocation.xRight
                     << ", " << rectlPageLocation.yTop
                     << ") }"
              << endl;
#endif

      PBYTE pbScanline = 0;
      int   cbScanline = rectlPageLocation.yTop - rectlPageLocation.yBottom + 1;

      // Read in the bitmap's scanlines
      for (int iScanLine = 0; iScanLine < cbScanline; iScanLine++)
      {
         pbScanline = pbBitmapBits + iScanLine * icbBitmapScanline;

         int iBitmapScanline = pbmi2Bitmap->cy
                             - ( rectlPageLocation.yBottom
                               + iScanLine
                               );

#ifndef RETAIL
/////////if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << dec << "DeviceTester6: fseek ([" << iBitmapScanline << "]" << -((iBitmapScanline+1)*icbBitmapScanline) << ") = ";
#endif

         // Seek to the scanline in the file
         rc = fseek (fp,
                     -( ( iBitmapScanline
                        + 1
                        )
                      * icbBitmapScanline
                      ),
                     SEEK_END);

#ifndef RETAIL
/////////if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << rc << endl << "DeviceTester6: fread (" << icbBitmapScanline << ") = ";
#endif

         rc = fread (pbScanline,
                     icbBitmapScanline,
                     1,
                     fp);

#ifndef RETAIL
/////////if (DebugOutput::shouldOutputDeviceTester6 ()) cerr << rc << endl;
#endif
      }

      // Send it to the device
      pDevice->rasterize (pbBitmapBits,
                          pbmi2Bitmap,
                          &rectlPageLocation,
                          BITBLT_BITMAP);
   }

done:
   // Clean up!
   if (pbmi2Bitmap)
      free (pbmi2Bitmap);

   if (pbBitmapBits)
      free (pbBitmapBits);
}

Device *
createDevice (char *pszDriverLibrary,
              char *pszJobProperties)
{
#define LIBDEVICENAME3(s) STRINGIZE(s)
#define LIBDEVICENAME2(x) LIBDEVICENAME3(lib##x##.so)
#define LIBDEVICENAME1(x) LIBDEVICENAME2(x)
#define LIBDEVICENAME     LIBDEVICENAME1(DEVICENAME)

   char               achLibDeviceName[512]; // @TBD - need max path define
   Device            *pDevice           = 0;
   PFNNEWDEVICE       pfnNewDevice      = 0;
   PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;

   if (  0 == pszDriverLibrary
      || !*pszDriverLibrary
      )
   {
      cerr << "DeviceTester6: pszDriverLibrary is not filled in!" << endl;
      return 0;
   }

   if (0 != strncasecmp (pszDriverLibrary, "lib", 3))
   {
      sprintf (achLibDeviceName, "lib%s.so", pszDriverLibrary);
   }
   else
   {
      strcpy (achLibDeviceName, pszDriverLibrary);
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << "DeviceTester6: Trying to load " << pszDriverLibrary << endl;
   }
#endif

   vhmodDevice = g_module_open (pszDriverLibrary, (GModuleFlags)0);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << "DeviceTester6: vhmodDevice = " << hex << vhmodDevice << endl;
   }
#endif

   if (!vhmodDevice)
   {
      cerr << "DeviceTester6: g_module_error returns " << g_module_error () << endl;
      return 0;
   }

   // nm libEpson_Stylus_Color_860.so | grep newDevice

   // returns 00011e20 T newDevice__Fv for
   // extern Device *newDevice (bool fAdvanced);
   g_module_symbol (vhmodDevice, "newDevice__Fb", (gpointer *)&pfnNewDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << hex << "DeviceTester6: pfnNewDevice = 0x" << (int)pfnNewDevice << endl;
   }
#endif

   if (!pfnNewDevice)
   {
      cerr << "DeviceTester6: g_module_error returns " << g_module_error () << endl;

      return 0;
   }

   // returns 00011d30 T newDevice__FPcb for
   // extern Device *newDevice (char *pszJobProperties, bool fAdvanced);
   g_module_symbol (vhmodDevice, "newDevice__FPcb", (gpointer *)&pfnNewDeviceWArgs);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << hex << "DeviceTester6: pfnNewDeviceWArgs = 0x" << (int)pfnNewDeviceWArgs << endl;
   }
#endif

   if (!pfnNewDeviceWArgs)
   {
      cerr << "DeviceTester6: g_module_error returns " << g_module_error () << endl;

      return 0;
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << "DeviceTester6: ********** Creating device with passed in properties **********" << endl;
      cerr << "DeviceTester6: The Properties are: " << pszJobProperties << endl;
   }
#endif

   pDevice = pfnNewDeviceWArgs (pszJobProperties, false);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << "DeviceTester6: pfnNewDeviceWArgs (...) returns " << *pDevice << endl;
   }
#endif

   return pDevice;
}

void
printUsage (char *pszExeName)
{
   cerr << "Usage: " << pszExeName << " required_parms optional_parms" << endl
        << "   where required_parms are:" << endl
        << "      bmp bitmapfile.bmp" << endl
        << "      driver libDriverName.so" << endl
        << "   where optional_parms are:" << endl
        << "      cout output_file" << endl
        << "      cerr error_file" << endl
        << "      scanlines number_of_scanlines" << endl
        << "      orientation orientation_id" << endl
        << "      form form_id" << endl
        << "      tray tray_id" << endl
        << "      media media_id" << endl
        << "      resolution resolution_id" << endl
        << "      dithering dithering_id" << endl
        << "      printmode printmode_id" << endl;
}

int
main (int argc, char *argv[])
{
#define MAX_SCAN_LINES 32
#define JP_BLOCK_SIZE  256
   ofstream *pfcout            = 0;
   ofstream *pfcerr            = 0;
   char     *pszCout           = 0;
   char     *pszCerr           = 0;
   char     *pszFilename       = 0;
   char     *pszDriverLibrary  = 0;
   char     *pszJobProperties  = 0;
   int       cbJobPropertyLeft = 0;
   int       iNumScanLines     = MAX_SCAN_LINES;
   FILE     *fp                = 0;

   if (!g_module_supported ())
   {
      cerr << "This program needs glib's module routines!" << endl;
      return 1;
   }

#ifdef DEBUG
   /* HACK:  Work around gdb debugging of a program that writes to both
   **        stdout and stderr.
   */
   pszCout = "output";
   pszCerr = "output.err";
#endif

   for (int i = 1; i < argc; i++)
   {
      if (0 == strcmp ("bmp", argv[i]))
      {
         pszFilename = argv[++i];
      }
      else if (0 == strcmp ("cout", argv[i]))
      {
         pszCout = argv[++i];
      }
      else if (0 == strcmp ("cerr", argv[i]))
      {
         pszCerr = argv[++i];
      }
      else if (0 == strcasecmp ("driver", argv[i]))
      {
         pszDriverLibrary = argv[++i];
      }
      else if (0 == strcasecmp ("scanlines", argv[i]))
      {
         iNumScanLines = atoi (argv[++i]);
      }
      else
      {
         // Assume that it is a Job Property (where there needs to be at least
         // 2 parameters)
         if (2 <= argc - i)
         {                                     // =                         ' \0'
            int iSpaceNeeded = strlen (argv[i]) + 1 + strlen (argv[i + 1]) + 2;

////////////cerr << "DeviceTester6: iSpaceNeeded = " << iSpaceNeeded << endl;

            // Out of space?
            if (  !pszJobProperties
               || cbJobPropertyLeft < iSpaceNeeded
               )
            {
               cbJobPropertyLeft = JP_BLOCK_SIZE;
               if (pszJobProperties)
                  cbJobPropertyLeft += strlen (pszJobProperties) + 1;

///////////////cerr << "DeviceTester6: Out of space! cbJobPropertyLeft = " << cbJobPropertyLeft << endl;

               char *pszTemp = (char *)calloc (1, cbJobPropertyLeft);

               if (!pszTemp)
               {
                  cerr << "DeviceTester6: Out of memory!" << endl;

                  return 1;
               }

               *pszTemp = '\0';

               if (pszJobProperties)
               {
                  strcpy (pszTemp, pszJobProperties);
                  free (pszJobProperties);
               }
               pszJobProperties = pszTemp;
            }

            strcat (pszJobProperties, argv[i]);
            strcat (pszJobProperties, "=");
            strcat (pszJobProperties, argv[i + 1]);
            strcat (pszJobProperties, " ");

            cbJobPropertyLeft -= strlen (argv[i]) + 1 + strlen (argv[i + 1]) + 2;

////////////cerr << "DeviceTester6: pszJobProperties = " << pszJobProperties << endl;
////////////cerr << "DeviceTester6: cbJobPropertyLeft = " << cbJobPropertyLeft << endl;

            i++;
         }
         else
         {
            printUsage (argv[0]);

            return 2;
         }
      }
   }

   // Check for required parameters
   if (  0 == pszFilename
      || 0 == pszDriverLibrary
      )
   {
      printUsage (argv[0]);

      return 3;
   }

   if (pszCout)
   {
      pfcout = new ofstream (pszCout, ios::out | ios::binary);
      cout   = *pfcout;
   }

   if (pszCerr)
   {
      pfcerr = new ofstream (pszCerr);
      cerr   = *pfcerr;
   }

   Device *pDevice = createDevice (pszDriverLibrary, pszJobProperties);

   if (!pDevice)
   {
      return 4;
   }

   string *pJobProp = pDevice->queryCurrentJobProperties ();

   cerr << "DeviceTester6: The device's job properties are: " << *pJobProp << endl;

   delete pJobProp;

   fp = fopen (pszFilename, "rb");
   if (fp)
   {
      USHORT usType = 0;
      int    rc;

      rc = fread (&usType, sizeof (usType), 1, fp);

      if (BFT_BMAP == usType)
      {
         pDevice->beginJob ();

         bitblt (fp, usType, iNumScanLines, pDevice);

         pDevice->endJob ();
      }
      else
      {
         cerr << hex << "DeviceTester6: Error: " << pszFilename << " is not a valid bitmap because usType = 0x" << usType << endl;
         return 5;
      }

      fclose (fp);
   }
   else
   {
      cerr << "DeviceTester6: Error: cannot open " << pszFilename << " for reading" << endl;
   }

   delete pDevice;

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << "DeviceTester6: In " << __FUNCTION__ << ", after delete pDevice;" << endl;
   }
#endif

   int rc = 0;

   rc = g_module_close (vhmodDevice);

#ifndef RETAIL
   if (DebugOutput::shouldOutputDeviceTester6 ())
   {
      cerr << dec << "DeviceTester6: dlclose returns " << rc << endl;
   }
#endif

   if (pfcout)
   {
      delete pfcout;
   }

   if (pfcerr)
   {
      delete pfcerr;
   }

   if (pszJobProperties)
   {
      free (pszJobProperties);
   }

   return 0;
}
