/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>

#include "OmniIJSProxy.hpp"

int
main (int argc, char *argv[])
{
   OmniIJSProxy *pDevice = 0;

   if (2 > argc)
   {
      cerr << "Usage: " << argv[0] << " printer_library_name [job_properties]" << endl;
      return 1;
   }

   if (getenv ("DEBUGOMNI"))
   {
      DebugOutput::setOutputOmniIJSProxy (true);
      DebugOutput::setOutputOmniServer (true);
      DebugOutput::setOutputPrinterCommand (true);
   }

   pDevice = new OmniIJSProxy (argv[1],
                               (3 <= argc ? argv[2] : 0));

   cout << "Driver version is : "
        << pDevice->getVersion ()
        << endl
        << "Driver name is : "
        << pDevice->getDriverName ()
        << endl
        << "Device name is : "
        << pDevice->getDeviceName ()
        << endl;

   cout << "The PDL level is "
        << pDevice->getPDLLevel ()
        << endl
        << "The PDL sublevel is "
        << pDevice->getPDLSubLevel ()
        << endl
        << "The PDL major revision level is "
        << pDevice->getPDLMajorRevisionLevel ()
        << endl
        << "The PDL minor revision level is "
        << pDevice->getPDLMinorRevisionLevel ()
        << endl;

   DeviceOrientation *pDO = pDevice->getCurrentOrientation ();

   cout << "The current orientation is " << *pDO << endl;

   char *pszDitherID = pDevice->getCurrentDitherID ();

   cout << "The current dither id is " << pszDitherID << endl;

   DeviceForm *pDF = pDevice->getCurrentForm ();

   cout << "The current form is " << *pDF << endl;

   DeviceTray *pDT = pDevice->getCurrentTray ();

   cout << "The current tray is " << *pDT << endl;

   DeviceMedia *pDM = pDevice->getCurrentMedia ();

   cout << "The current media is " << *pDM << endl;

   DeviceResolution *pDR = pDevice->getCurrentResolution ();

   cout << "The current resolution is " << *pDR << endl;

   DevicePrintMode *pDPM = pDevice->getCurrentPrintMode ();

   cout << "The current print mode is " << *pDPM << endl;

   Enumeration *pEnum = pDO->getEnumeration ();

   cout << "Portrait is ";
   if (!pDO->isOrientationSupported (DeviceOrientation::ORIENTATION_PORTRAIT))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "Letter is ";
   if (!pDF->isFormSupported (DeviceForm::FORM_LETTER))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "Auto is ";
   if (!pDT->isTraySupported (DeviceTray::TRAY_AUTO))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "Plain is ";
   if (!pDM->isMediaSupported (DeviceMedia::MEDIA_PLAIN))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "360x360 is ";
   if (!pDR->isResolutionSupported (DeviceResolution::RESOLUTION_360_X_360))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "CMYK is ";
   if (!pDPM->isPrintModeSupported (DevicePrintMode::PRINT_MODE_24_CMYK))
   {
      cout << "not ";
   }
   cout << "supported" << endl;

   cout << "Enumerating orientations:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDO = (DeviceOrientation *)pEnum->nextElement ();

      cout << DeviceOrientation::IDToName (pDO->getID ()) << endl;

      delete pDO;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDF->getEnumeration ();

   cout << "Enumerating forms:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDF = (DeviceForm *)pEnum->nextElement ();

      cout << DeviceForm::IDToName (pDF->getID ()) << endl;

      delete pDF;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDT->getEnumeration ();

   cout << "Enumerating trays:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDT = (DeviceTray *)pEnum->nextElement ();

      cout << DeviceTray::IDToName (pDT->getID ()) << endl;

      delete pDT;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDM->getEnumeration ();

   cout << "Enumerating medias:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDM = (DeviceMedia *)pEnum->nextElement ();

      cout << DeviceMedia::IDToName (pDM->getID ()) << endl;

      delete pDM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDR->getEnumeration ();

   cout << "Enumerating resolutions:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDR = (DeviceResolution *)pEnum->nextElement ();

      cout << DeviceResolution::IDToName (pDR->getID ()) << endl;

      delete pDR;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDPM->getEnumeration ();

   cout << "Enumerating print modes:" << endl;
   while (pEnum->hasMoreElements ())
   {
      pDPM = (DevicePrintMode *)pEnum->nextElement ();

      cout << DevicePrintMode::IDToName (pDPM->getID ()) << endl;

      delete pDPM;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->getDitherEnumeration ();

   cout << "Enumerating dither ids:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszID = (char *)pEnum->nextElement ();

      cout << pszID << endl;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->listJobPropertyKeys ();

   cout << "Enumerating job property types:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszKey = (char *)pEnum->nextElement ();

      if (!pszKey)
      {
         cerr << "Error: Enumeration returns null!" << endl;
         break;
      }

      string *pstringValue = pDevice->getJobPropertyType (pszKey);

      if (!pstringValue)
      {
         cerr << "Error: getJobPropertyType (" << pszKey << ") return null!" << endl;
         break;
      }

      cout << pszKey << " = " << *pstringValue << endl;

      delete pstringValue;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->listDeviceJobPropertyKeys ();

   cout << "Enumerating device job property types:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszKey = (char *)pEnum->nextElement ();

      if (!pszKey)
      {
         cerr << "Error: Enumeration returns null!" << endl;
         break;
      }

      string *pstringValue = pDevice->getJobPropertyType (pszKey);

      if (!pstringValue)
      {
         cerr << "Error: getJobPropertyType (" << pszKey << ") return null!" << endl;
         break;
      }

      cout << pszKey << " = " << *pstringValue << endl;

      delete pstringValue;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->listJobPropertyKeys ();

   cout << "Enumerating job properties:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszKey = (char *)pEnum->nextElement ();

      if (!pszKey)
      {
         cerr << "Error: Enumeration returns null!" << endl;
         break;
      }

      string *pstringValue = pDevice->getJobProperty (pszKey);

      if (!pstringValue)
      {
         cerr << "Error: getJobProperty (" << pszKey << ") return null!" << endl;
         break;
      }

      cout << pszKey << " = " << *pstringValue << endl;

      delete pstringValue;
   }
   cout << "Done!" << endl;

   delete pEnum;

   pEnum = pDevice->listDeviceJobPropertyKeys ();

   cout << "Enumerating device job properties:" << endl;
   while (pEnum->hasMoreElements ())
   {
      char *pszKey = (char *)pEnum->nextElement ();

      if (!pszKey)
      {
         cerr << "Error: Enumeration returns null!" << endl;
         break;
      }

      string *pstringValue = pDevice->getJobProperty (pszKey);

      if (!pstringValue)
      {
         cerr << "Error: getJobProperty (" << pszKey << ") return null!" << endl;
         break;
      }

      cout << pszKey << " = " << *pstringValue << endl;

      delete pstringValue;
   }
   cout << "Done!" << endl;

   delete pEnum;

   if (pDevice->hasCapability (Capability::COLOR))
   {
      cout << "This device can print in color." << endl;
   }

   if (pDevice->hasCapability (Capability::MONOCHROME))
   {
      cout << "This device can print in monochrome." << endl;
   }

   if (pDevice->hasCapability (Capability::MIRROR))
   {
      cout << "This device can print in mirror image." << endl;
   }

   delete pDevice;

   sleep (1);

   return 0;
}
