/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "GhostscriptInterface.hpp"
#include <iostream.h>
#include <fstream.h>

#include <glib.h>
#include <gmodule.h>

/*------------------------------------------------------------------------*/
/*  GetResolutionInfo                                                     */
/*------------------------------------------------------------------------*/
bool
GetResolutionInfo (void *pDev, HWRESOLUTION * hwRes)
{
   bool              bRet    = false;

   DeviceResolution *pDevRes;

   if(bIJSDevice)
       pDevRes = pIJSDevice_d->getCurrentResolution();  // Device Resolution
   else
       pDevRes = pDevice_d->getCurrentResolution();  // Device Resolution


#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << ", hwRes = 0x" << (int)hwRes << dec << endl;
#endif

   if (pDev && pDevRes)
   {
      hwRes->xRes = pDevRes->getXRes();
      hwRes->yRes = pDevRes->getYRes();
      hwRes->fScanDots = pDevRes->getScanlineMultiple();
      bRet = true;
   }

   return bRet;
}

/*------------------------------------------------------------------------*/
/*   GetMarginInfo                                                        */
/*------------------------------------------------------------------------*/
bool
GetMarginInfo (void *pDev, HWMARGINS * hwMargins)
{
   bool         bRet    = false;
   HardCopyCap *pHCC;  // Hard copy information

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << ", hwMargins = 0x" << (int)hwMargins << dec << endl;
#endif

   if(bIJSDevice)
       pHCC = pIJSDevice_d->getCurrentForm ()->getHardCopyCap ();
   else
       pHCC = pDevice_d->getCurrentForm ()->getHardCopyCap ();

   if (pHCC)
   {
      hwMargins->fxWidth     = pHCC->getXPels ();
      hwMargins->fyHeight    = pHCC->getYPels ();

      hwMargins->fLeftClip   = 0; // changed since Ghostscript needs to render full size of
      hwMargins->fTopClip    = 0; // the page printable area.  If we add in the margin, we
      hwMargins->fRightClip  = 0; // actually force GS to remove the margin from the printable
      hwMargins->fBottomClip = 0; // area.

//      hwMargins->fLeftClip   = pHCC->getLeftClip ();
//      hwMargins->fTopClip    = pHCC->getTopClip ();
//      hwMargins->fRightClip  = pHCC->getRightClip ();
//      hwMargins->fBottomClip = pHCC->getBottomClip ();
      bRet = true;
   }

   return bRet;
}

/*--------------------------------------------------------------------------------*/
/*                          GetPrintModeInfo                                      */
/*--------------------------------------------------------------------------------*/
bool
GetPrintModeInfo (void *pDev, PRINTMODE *pPrtMode)
{
   bool             bRet        = false;

   DevicePrintMode *pDevPrtMode;

   if(bIJSDevice)
       pDevPrtMode = pIJSDevice_d->getCurrentPrintMode ();
   else
       pDevPrtMode = pDevice_d->getCurrentPrintMode ();


#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << ", pPrtMode = 0x" << (int)pPrtMode << dec << endl;
#endif

   if (pDev && pPrtMode && pDevPrtMode)
   {
      pPrtMode->iBitCount = pDevPrtMode->getLogicalCount ();
      pPrtMode->iPlanes   = pDevPrtMode->getNumPlanes ();
      bRet = true;
   }

   return bRet;
}

/*--------------------------------------------------------------------------------*/
/*                          SetOutputFunction                                     */
/*--------------------------------------------------------------------------------*/
void
SetOutputFunction (void *pDev, PFNOUTPUTFUNCTION pfn, void *pMC)
{
     pDevice_d->setOutputFunction (pfn, pMC);
}


/*--------------------------------------------------------------------------------*/
/*                          BeginJob                                              */
/*--------------------------------------------------------------------------------*/
void *
BeginJob (void *pDev, FILE *pFile)
{

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << ", pFile = 0x" << (int)pFile << dec << endl;
#endif

///myStream *OutStream = new myStream ();

///OutStream->setFileHandle (pFile);

///OutStream->setName ("WriterClass");

///pDevice->setOutputStream (OutStream);

   if(bIJSDevice)
       pIJSDevice_d->beginJob ();
   else
   {
       pDevice_d->setOutputStream (fileno (pFile));
       pDevice_d->beginJob ();
   }


   return (void *)NULL;
}

/*--------------------------------------------------------------------------------*/
/*                          BeginJob                                              */
/*--------------------------------------------------------------------------------*/
void
BeginJob (void *pDev)
{

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << dec << endl;
#endif

   if(bIJSDevice)
       pIJSDevice_d->beginJob ();
   else
       pDevice_d->beginJob ();
}

/*--------------------------------------------------------------------------------*/
/*                          Rasterize                                             */
/*--------------------------------------------------------------------------------*/
void
Rasterize (void             *pDev,
           PBYTE             pbBits,
           PBITMAPINFO2      pbmi,
           PSIZEL            psizelPage,
           PRECTL            prectlPageLocation,
           BITBLT_TYPE       eType)
{

   if(bIJSDevice)
       pIJSDevice_d->rasterize (pbBits,
                                pbmi,
                                prectlPageLocation,
                                eType);
   else
       pDevice_d->rasterize (pbBits,
                             pbmi,
                             prectlPageLocation,
                             eType);
}

/*--------------------------------------------------------------------------------*/
/*                          NewPage                                               */
/*--------------------------------------------------------------------------------*/
void
NewFrame (void *pDev)
{

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << dec << endl;
#endif

   if(bIJSDevice)
       pIJSDevice_d->newFrame ();
   else
       pDevice_d->newFrame ();
}

/*--------------------------------------------------------------------------------*/
/*                          EndJob                                                */
/*--------------------------------------------------------------------------------*/
void
EndJob (void *pDev, void *pStream)
{
   ofstream *outstream = (ofstream *)pStream;

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << ", pStream " << (int)pStream << dec << endl;
#endif

   if(bIJSDevice)
       pIJSDevice_d->endJob ();
   else
       pDevice_d->endJob ();

   if(pIJSDevice_d)
   {
       delete pIJSDevice_d;
   }

   delete outstream;
}

/*--------------------------------------------------------------------------------*/
/*                          EndJob                                                */
/*--------------------------------------------------------------------------------*/
void
EndJob (void *pDev)
{

#ifndef RETAIL
   if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDev = 0x" << hex << (int)pDev << dec << endl;
#endif

   if(bIJSDevice)
       pIJSDevice_d->endJob ();
   else
       pDevice_d->endJob ();
}

/*--------------------------------------------------------------------------------*/
/*                            CreateDevice                                        */
/*--------------------------------------------------------------------------------*/
void *
createDevice (char  *pszDeviceName,
              void  *pOutputObject,
              void **pvhDevice,
              char  *pszCerr,
              char  *pszJobProperties,
              int    iUseClient,
              FILE  *pFileIn)
{
   if(iUseClient)
   {
       pIJSDevice_d = new OmniIJSProxy (pszDeviceName, pszJobProperties, true, fileno (pFileIn));
       bIJSDevice = true;
       return((void *)pIJSDevice_d);
   }
   else
   {
      if (!g_module_supported ())
      {
         cerr << "This program needs glib's module routines!" << endl;
         return 0;
      }

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << endl;
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pszDeviceName    = " << pszDeviceName << endl;
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pOutputObject    = 0x" << hex << (int)pOutputObject << endl;
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pvhDevice        = 0x" << (int)pvhDevice << endl;
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pszCerr          = " << pszCerr << endl;
       if (DebugOutput::shouldOutputOmniInterface ()) cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pszJobProperties = " << pszJobProperties << dec << endl;
#endif

#define LIBDEVICENAME3(s) STRINGIZE(s)
#define LIBDEVICENAME2(x) LIBDEVICENAME3(lib##x##.so)
#define LIBDEVICENAME1(x) LIBDEVICENAME2(x)
#define LIBDEVICENAME     LIBDEVICENAME1(DEVICENAME)

typedef Device * (*PFNNEWDEVICEWARGS) (char *pszJobProperties, bool fAdvanced);
typedef Device * (*PFNNEWDEVICE)      (bool fAdvanced);

       char               cDeviceName[64];
       PFNNEWDEVICE       pfnNewDevice      = 0;
       PFNNEWDEVICEWARGS  pfnNewDeviceWArgs = 0;

       pDevice_d  = 0;

       bIJSDevice = false;

       // Added for debugging convienence
       if (  pszCerr
          && pszCerr[0]
          )
       {
          ofstream *fcerr = new ofstream (pszCerr);
          cerr = *fcerr;
          pOutputObject = (void *)fcerr;
       }

       if (0 != strncmp (pszDeviceName, "lib", 3))
          sprintf (cDeviceName, "lib%s.so", pszDeviceName);
       else
          strcpy (cDeviceName, pszDeviceName);

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ())
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": Trying to load " << cDeviceName << endl;
#endif

       char *pszDeviceLib = 0;
       pszDeviceLib = (char *) malloc(strlen(cDeviceName)+strlen("/opt/Omni/bin/")+1);
       if(pszDeviceLib)
           sprintf(pszDeviceLib, "/opt/Omni/bin/%s", cDeviceName);
       else
           return 0;
       *pvhDevice = g_module_open (pszDeviceLib, (GModuleFlags)0);

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ())
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": *pvhDevice = " << hex << *pvhDevice << dec << endl;
#endif

       if(pszDeviceLib)
           free(pszDeviceLib);

       if (!*pvhDevice)
       {
          *pvhDevice = g_module_open (cDeviceName, (GModuleFlags)0);

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ())
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": *pvhDevice = " << hex << *pvhDevice << dec << endl;
#endif

          if (!*pvhDevice)
          {
              cerr << endl << "<<<<<<<<<<<<<<<<<<<<<< ERROR >>>>>>>>>>>>>>>>>>>>>>>" << endl;
              cerr << endl << endl;
              cerr << "GhostscriptInterface::" << __FUNCTION__ << ": g_module_error returns " << g_module_error () << endl;
              cerr << "GhostscriptInterface::" << __FUNCTION__ << ": cDeviceName = " << cDeviceName << endl;
              cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pszDeviceName = " << pszDeviceName << endl;
              cerr << endl << endl;


              return 0;
          }
       }

       // nm libHP_Deskjet_1120Cxi.so

       // returns 00011e20 T newDevice__Fv for
       // extern Device *newDevice (bool fAdvanced);
       g_module_symbol ((GModule *)*pvhDevice, "newDevice__Fb", (gpointer *)&pfnNewDevice);

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ())
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pfnNewDevice = 0x" << hex << (int)pfnNewDevice << endl;
#endif

       if (!pfnNewDevice)
       {
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": g_module_error returns " << dec << g_module_error () << endl;

          return 0;
       }

       // returns 00011d30 T newDevice__FPcb for
       // extern Device *newDevice (char *pszJobProperties, bool fAdvanced);
       g_module_symbol ((GModule *)*pvhDevice, "newDevice__FPcb", (gpointer *)&pfnNewDeviceWArgs);

#ifndef RETAIL
       if (DebugOutput::shouldOutputOmniInterface ())
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pfnNewDeviceWArgs = 0x" << hex << (int)pfnNewDeviceWArgs << endl;
#endif

       if (!pfnNewDeviceWArgs)
       {
          cerr << "GhostscriptInterface::" << __FUNCTION__ << ": g_module_error returns " << dec << g_module_error () << endl;

          return 0;
       }

       if(pszJobProperties && pszJobProperties[0])
           pDevice_d = pfnNewDeviceWArgs (pszJobProperties, true);
       else
           pDevice_d = pfnNewDevice (true);

#ifndef RETAIL
          if (DebugOutput::shouldOutputOmniInterface ())
             cerr << "GhostscriptInterface::" << __FUNCTION__ << ": pDevice = " << *pDevice_d << endl;
#endif

       return pDevice_d;
   }

   return 0;

}
