/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _OmniInterface
#define _OmniInterface

#include "OmniIJSProxy.hpp"
#include "defines.hpp"
#include "Device.hpp"
#include <stdio.h>

OmniIJSProxy *pIJSDevice_d;
Device       *pDevice_d;
bool          bIJSDevice;


typedef struct _HWMAR
{
  float fLeftClip;
  float fBottomClip;
  float fRightClip;
  float fTopClip;
  float fxWidth;
  float fyHeight;
} HWMARGINS;

typedef struct _HWRES
{
  float xRes;
  float yRes;
  float fScanDots;  // number of dots in scan line
} HWRESOLUTION;

typedef struct _PRTMODE
{
  int iBitCount;
  int iPlanes;
} PRINTMODE;

extern "C" {

bool GetResolutionInfo( void *pDev, HWRESOLUTION * hwRes);

bool GetMarginInfo( void *pDev, HWMARGINS * hwMargins);

bool GetPrintModeInfo(void *pDev, PRINTMODE *pPrtMode);

void * BeginJob(void *pDev, FILE *pfile);

void NewFrame (void *pDev);

void EndJob(void *pDev, void *pStream);

void Rasterize(void *pDevice,
               PBYTE             pbBits,
               PBITMAPINFO2      pbmi,
               PSIZEL            psizelPage,
               PRECTL            prectlPageLocation,
               BITBLT_TYPE       eType);

void *  createDevice (char  *pszDeviceName,
                      void  *pOutputObject,
                      void **vhDevice,
                      char  *pszCerr,
                      char  *pszOtherOpts,
                      int    iUseClient,
                      FILE  *pfileIn);

}

#endif
