/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2001
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _HP_LaserJet_PCL_Instance
#define _HP_LaserJet_PCL_Instance

#include "defines.hpp"
#include "DeviceInstance.hpp"
#include "DeviceGamma.hpp"

class HP_LaserJet_PCL_Instance : public DeviceInstance
{
public:
   enum {
      PT_AUTO_ROTATE,
      PT_SCALABLE_FONT,
      PT_SUPPORT_DUPLEX,
      PT_SUPPORT_RULES,
      PT_SUPPORT_DOT_COORDS,
      PT_SUPPORT_PJL,
      PT_SUPPORT_BACKBIN,
      PT_SUPPORT_JOBOFFSET,
      PT_SUPPORT_PATTERNS,
      PT_SUPPORT_PCL5,
      PT_MIXED_TRAY,
      PT_SUPPORT_COLOR,
      PT_SUPPORT_PS,
      PT_SUPPORT_PAGEPROT_ESC,
      PT_SUPPORT_ES,
      PT_SUPPORT_HPGL2,
      PT_SUPPORT_HPGL2_MC,
      PT_PAGEPROT_AUTO,
      PT_SUPPORT_COMPRESS5,
      PT_SUPPORT_COLORSMART,
      PT_SUPPORT_HPGL_DEFAULT,
      PT_SUPPORT_OUTPUTOPTION,
      PT_PRINTER_DISPLAY,
      PT_SUPPORT_COLOR_RET,
      PT_SUPPORT_HGRAYSCALE,
      PT_SUPPORT_WATERMARK,
      PT_SUPPORT_HPGL2MACRO,
      PT_SUPPORT_PAN_SUPER,
      PT_SUPPORT_CONFIG,
      PT_SUPPORT_HPGL2LOW,
      PT_SUPPORT_BIDI,
      PT_SUPPORT_STAPLER,
      PT_SUPPORT_PCLXL,
      PT_SUPPORT_AUTOSELECT,
      PT_SUPPORT_PCLXL_DEFAULT,
      PT_SUPPORT_PCLXL_1200,
      PT_SUPPORT_LARGECLRFMT,
      PT_SUPPORT_2STAPLES,
      PT_SUPPORT_PCLXL_600,
      PT_SUPPORT_NEWCLR,
      PT_SUPPORT_PJLOUTPUTBIN,
      PT_SUPPORT_COLLATOR,
      PT_SUPPORT_MET,
      PT_SUPPORT_MOPIER,
      PT_SUPPORT_MOPIER_DEFAULT,
      PT_SUPPORT_EUROCHAR,
      PT_SUPPORT_JOB_RETENTION,
      PT_SUPPORT_PCLXL_1200R,
      PT_SUPPORT_FAX,
      PT_SUPPORT_SORTER,
      PT_SUPPORT_COLOR_OEM,
      PT_SUPPORT_COLLATE,
      PT_SUPPORT_QUIETRESET,
      PT_SUPPORT_EDGEPRINTING,
      PT_SUPPORT_OPTIMIZEPQ,
      PT_SUPPORT_COLORSMART2,
      PT_SUPPORT_MANUALDUPLEX,
      PT_SUPPORT_NOFNTMGR,
      PT_SUPPORT_PRINTERSELECTDEF,
      PT_SUPPORT_PCL6_REV20,
      PT_SUPPORT_PCL6_INT
   };

                     HP_LaserJet_PCL_Instance  (Device                         *pDevice);
   virtual          ~HP_LaserJet_PCL_Instance  ();

   void              initializeInstance        ();

   Enumeration      *listDeviceJobPropertyKeys ();
   bool              handleDeviceJobProperty   (char                           *pszKey,
                                                char                           *pszValue);
   string           *getJobPropertyType        (char                           *pszKey);
   string           *getJobProperty            (char                           *pszKey);

   virtual bool      beginJob                  ();
   virtual bool      newFrame                  ();
   virtual bool      newFrame                  (bool                            fJobPropertiesChanged);
   virtual bool      endJob                    ();
   virtual bool      abortJob                  ();

   virtual string    toString                  (ostrstream&                     oss);
   friend ostream&   operator<<                (ostream&                        os,
                                                const HP_LaserJet_PCL_Instance& device);

   void              setupPrinter              ();

   POINTL            ptlPrintHead_d;
   int               iUOM_d;
   int               iXScalingFactor_d;
   int               iYScalingFactor_d;
   int               iYUOMScalingFactor_d;
   int               iVerticalOffset_d;

private:
   bool              fHaveInitialized_d;
   bool              fHaveSetupPrinter_d;
   int               iHardwareScaling_d;
};

#endif
