/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "KS_ESC_Instance.hpp"
#include "defines.hpp"

#include <iostream>
#include <strstream>

const static bool fTestNoCompression = true;

KS_ESC_Instance::
KS_ESC_Instance (Device *pDevice)
   : DeviceInstance (pDevice)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::KS_ESC_Instance ()" << endl;
#endif

   fHaveInitialized_d  = false;
   fHaveSetupPrinter_d = false;
   fUseBidirectional_d = true;
   ptlPrintHead_d.x    = 0;
   ptlPrintHead_d.y    = 0;
}

KS_ESC_Instance::
~KS_ESC_Instance ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::~KS_ESC_Instance ()" << endl;
#endif
}

void KS_ESC_Instance::
initializeInstance ()
{
   if (fHaveInitialized_d)
      return;

   fHaveInitialized_d = true;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::initializeInstance ()" << endl;
#endif
}

static char *apszDeviceJobPropertyKeys[] = {
   "bidirectional"
};
#define JOBPROP_BIDIRECTIONAL apszDeviceJobPropertyKeys[0]

Enumeration * KS_ESC_Instance::
listDeviceJobPropertyKeys ()
{
   class DeviceJobPropertyKeyEnumerator : public Enumeration
   {
   public:
      DeviceJobPropertyKeyEnumerator (char **apszKeys, int iDimOfKeys)
      {
         iWhichKey_d = iDimOfKeys;
         apszKeys_d  = apszKeys;
      }

      virtual bool hasMoreElements ()
      {
         return iWhichKey_d > 0;
      }

      virtual void * nextElement ()
      {
         if (0 < iWhichKey_d)
         {
            iWhichKey_d--;
            return apszKeys_d[iWhichKey_d];
         }

         return 0;
      }

   private:
      int    iWhichKey_d;
      char **apszKeys_d;
   };

   return new DeviceJobPropertyKeyEnumerator (apszDeviceJobPropertyKeys, dimof (apszDeviceJobPropertyKeys));
}

bool KS_ESC_Instance::
handleDeviceJobProperty (char *pszKey,
                         char *pszValue)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::handleDeviceJobProperty ("
                          << pszKey << ", "
                          << pszValue << ")"
                          << endl;
#endif

   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      if (0 == strcmp (pszValue, "true"))
      {
         fUseBidirectional_d = true;
         return true;
      }
      else if (0 == strcmp (pszValue, "false"))
      {
         fUseBidirectional_d = false;
         return true;
      }
   }

   return false;
}

string * KS_ESC_Instance::
getJobPropertyType (char *pszKey)
{
   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      if (fUseBidirectional_d)
      {
         return new string ("boolean true");
      }
      else
      {
         return new string ("boolean false");
      }
   }

   return 0;
}

string * KS_ESC_Instance::
getJobProperty (char *pszKey)
{
   if (0 == strcasecmp (pszKey, JOBPROP_BIDIRECTIONAL))
   {
      if (fUseBidirectional_d)
      {
         return new string ("true");
      }
      else
      {
         return new string ("false");
      }
   }

   return 0;
}

void KS_ESC_Instance::
setupPrinter ()
{
   if (fHaveSetupPrinter_d)
      return;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::setupPrinter ()" << endl;
#endif

   fHaveSetupPrinter_d = true;

   DeviceResolution *pRes      = getCurrentResolution ();
   DeviceForm       *pDF       = getCurrentForm ();
   HardCopyCap      *pHCC      = pDF->getHardCopyCap ();
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

   pCmd = pCommands->getCommandData ("cmdBidi");
   if (pCmd)
   {
      char chState = '0';

      if (fUseBidirectional_d)
      {
         chState = '1';
      }

      pDevice_d->sendPrintfToDevice (pCmd, chState);
   }
   else
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "Error: There is no cmdBidi defined for this device!" << endl;
#endif
   }

   // @TBD
///pDevice_d->sendBinaryDataToDevice (getCurrentTray ());

   pCmd = pCommands->getCommandData ("cmdSetColor");
   if (pCmd)
   {
      // Start with black ink (default)
      pDevice_d->sendPrintfToDevice (pCmd, '0');
   }

   int iLines = (int)(6.0 * (float)pHCC->getCy () / 25400.0);

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "iLines = " << iLines << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdSetSixthLineSpacing");
   if (pCmd)
   {
      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdSetPageLengthInLines");
   if (pCmd)
   {
      pDevice_d->sendPrintfToDevice (pCmd, iLines);
   }

   int iColumnSize = pRes->getScanlineMultiple ();
   int iSpacing    = 0;

   pCmd = pCommands->getCommandData ("cmdSetLineSpacing216inch");
   if (pCmd)
   {
      iSpacing = 216;
   }
   else
   {
      pCmd = pCommands->getCommandData ("cmdSetLineSpacing180inch");
      if (pCmd)
      {
         iSpacing = 180;
      }
   }

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << *pCmd << " iSpacing = " << iSpacing << ", iColumnSize = " << iColumnSize << ", pRes->getYRes () = " << pRes->getYRes () << endl;
#endif

   if (0 < iSpacing)
   {
      pDevice_d->sendPrintfToDevice (pCmd,
                                     iSpacing * iColumnSize / pRes->getYRes ());
   }
}

bool KS_ESC_Instance::
beginJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::beginJob ()" << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdInit");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdInit = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool KS_ESC_Instance::
beginJob (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::beginJob (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The print mode is = " << getCurrentPrintMode () << endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return beginJob ();
}

bool KS_ESC_Instance::
newFrame ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::newFrame ()" << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPageEject = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   ditherNewFrame ();

   ptlPrintHead_d.x = 0;
   ptlPrintHead_d.y = 0;

   return true;
}

bool KS_ESC_Instance::
newFrame (bool fJobPropertiesChanged)
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::newFrame (with props)" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The job properties are as follows:" << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The orientation is = " << getCurrentOrientation() << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "iDitherID = " << getCurrentDitherID () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The form is = " << getCurrentForm () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The tray is = " << getCurrentTray () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The media is = " << getCurrentMedia () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The resolution is = " << getCurrentResolution () << endl;
   if (DebugOutput::shouldOutputInstance ()) cerr << "The print mode is = " << getCurrentPrintMode () << endl;
#endif

   // @TBD - reinitialize with new job properties

   // Call common code
   return newFrame ();
}

bool KS_ESC_Instance::
endJob ()
{
   DeviceCommand    *pCommands = getCommands ();
   BinaryData       *pCmd      = 0;

#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::endJob ()" << endl;
#endif

   pCmd = pCommands->getCommandData ("cmdPageEject");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdPageEject = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   pCmd = pCommands->getCommandData ("cmdTerm");
   if (pCmd)
   {
#ifndef RETAIL
      if (DebugOutput::shouldOutputInstance ()) cerr << "cmdTerm = " << *pCmd << endl;
#endif

      pDevice_d->sendBinaryDataToDevice (pCmd);
   }

   return true;
}

bool KS_ESC_Instance::
abortJob ()
{
#ifndef RETAIL
   if (DebugOutput::shouldOutputInstance ()) cerr << "KS_ESC_Instance::endJob ()" << endl;
#endif

   int cx = getCurrentForm ()->getHardCopyCap ()->getXPels ();

   // The largest size is 24 lines
   int iSize = 24 * (cx + 7) / 8;

   PBYTE abData = new BYTE [iSize];

   memset (abData, 0, sizeof (abData));

   BinaryData data (abData, sizeof (abData));

   pDevice_d->sendBinaryDataToDevice (&data);

   delete []abData;

   return true;
}

string KS_ESC_Instance::
toString (ostrstream& oss)
{
   oss << hex
       << "{ "
       << DeviceInstance::toString (oss) // Add parent's output as well
       << " }"
       << dec
       << ends;

   return oss.str ();
}

/* Provide a way to print out class data
*/
ostream&
operator<< (ostream& os, const KS_ESC_Instance& const_self)
{
   KS_ESC_Instance& self = const_cast<KS_ESC_Instance&>(const_self);
   ostrstream          oss;

   os << self.toString (oss);

   return os;
}
