/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include <OkiPage_6e_6ex.hpp>

Device *
newDevice (bool fAdvanced)
{
   PrintDevice *pPrintDevice = new OkiPage_6e_6ex ();

   pPrintDevice->initialize ();

   if (fAdvanced)
      return pPrintDevice;
   else
      return new OmniProxy (pPrintDevice);
}

Device *
newDevice (char *pszJobProperties, bool fAdvanced)
{
   PrintDevice *pPrintDevice = new OkiPage_6e_6ex (pszJobProperties);

   pPrintDevice->initialize ();

   if (fAdvanced)
      return pPrintDevice;
   else
      return new OmniProxy (pPrintDevice);
}

void
deleteDevice (Device *pDevice)
{
   delete pDevice;
}

void OkiPage_6e_6ex::
commonInit ()
{
   setCapabilities ( Capability::MIRROR
                   | Capability::MONOCHROME
                   );

   setRasterCapabilities ( TOP_TO_BOTTOM
                         );

   setDeviceOptions ( Okidata_PCL_Instance::PT_AUTO_ROTATE
                    | Okidata_PCL_Instance::PT_SCALABLE_FONT
                    | Okidata_PCL_Instance::PT_SUPPORT_DOT_COORDS
                    | Okidata_PCL_Instance::PT_SUPPORT_HPGL2
                    | Okidata_PCL_Instance::PT_SUPPORT_PCL5
                    | Okidata_PCL_Instance::PT_SUPPORT_RULES
                    | Okidata_PCL_Instance::PT_SUPPORT_WATERMARK
                    );

   setDeviceInstance (new Okidata_PCL_Instance (this));
   setDeviceBlitter (new Okidata_PCL_Blitter (this));
   setPDL (new PDL (PDL::PDL_PCL,PDL::LEVEL_PCL3,1,0));
}

OkiPage_6e_6ex::
OkiPage_6e_6ex ()
   : PrintDevice ("Okidata",
                  "OkiPage 6e/6ex",
                  "OkiPage_6e_6ex",
                  "orientation=ORIENTATION_PORTRAIT dither=DITHER_STUCKI_DIFFUSION form=FORM_LETTER tray=TRAY_TRAY_1 media=MEDIA_PLAIN resolution=RESOLUTION_600_X_600 printmode=PRINT_MODE_1_ANY")
{
   commonInit ();
}

OkiPage_6e_6ex::
OkiPage_6e_6ex (char *pszJobProperties)
   : PrintDevice ("Okidata",
                  "OkiPage 6e/6ex",
                  "OkiPage_6e_6ex",
                  pszJobProperties)
{
   commonInit ();
}

DeviceGamma * OkiPage_6e_6ex::
getCurrentGamma ()
{
   return 0;
}

DeviceOrientation * OkiPage_6e_6ex::
getDefaultOrientation ()
{
   return Okidata_Orientations::create (Okidata_Orientations::ORIENTATION_PORTRAIT);
}

char * OkiPage_6e_6ex::
getDefaultDitherID ()
{
   return "DITHER_STUCKI_DIFFUSION";
}

DeviceForm * OkiPage_6e_6ex::
getDefaultForm ()
{
   return Okidata_OL810ex_Forms::create (Okidata_OL810ex_Forms::FORM_LETTER);
}

DeviceTray * OkiPage_6e_6ex::
getDefaultTray ()
{
   return Okidata_OL400ex_Trays::create (Okidata_OL400ex_Trays::TRAY_TRAY_1);
}

DeviceMedia * OkiPage_6e_6ex::
getDefaultMedia ()
{
   return Okidata_OL400ex_Medias::create (Okidata_OL400ex_Medias::MEDIA_PLAIN);
}

DeviceResolution * OkiPage_6e_6ex::
getDefaultResolution ()
{
   return Okidata_OL810ex_Resolutions::create (Okidata_OL810ex_Resolutions::RESOLUTION_600_X_600);
}

DeviceCommand * OkiPage_6e_6ex::
getDefaultCommands ()
{
   return new Okidata_PCL_Commands ();
}

DeviceData * OkiPage_6e_6ex::
getDefaultData ()
{
   return 0;
}

DevicePrintMode * OkiPage_6e_6ex::
getDefaultPrintMode ()
{
   return Okidata_PCL_PrintModes::create (Okidata_PCL_PrintModes::PRINT_MODE_1_ANY);
}
