/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Idea borrowed from http://hpinkjet.sourceforge.net
 */
#ifndef _OmniIJSProxy
#define _OmniIJSProxy

#include <unistd.h>
#include "Device.hpp"
#include "PrinterCommand.hpp"

class OmniIJSProxy
{
public:
                      OmniIJSProxy              (char              *pszDeviceName,
                                                 char              *pszJobProperties,
                                                 bool               fAdvanced = false,
                                                 int                fdStdOut  = STDOUT_FILENO,
                                                 int                fdStdErr  = STDERR_FILENO);
                     ~OmniIJSProxy              ();

   char              *getVersion                ();

   char              *getDriverName             ();
   char              *getDeviceName             ();
   char              *getShortName              ();

   DeviceOrientation *getCurrentOrientation     ();
   char              *getCurrentDitherID        ();
   DeviceForm        *getCurrentForm            ();
   DeviceTray        *getCurrentTray            ();
   DeviceMedia       *getCurrentMedia           ();
   DeviceResolution  *getCurrentResolution      ();
   DevicePrintMode   *getCurrentPrintMode       ();

   int                getPDLLevel               ();
   int                getPDLSubLevel            ();
   int                getPDLMajorRevisionLevel  ();
   int                getPDLMinorRevisionLevel  ();

   bool               hasCapability             (long               lMask);
   bool               hasRasterCapability       (long               lMask);
   bool               hasDeviceOption           (long               lMask);

   int                getScanlineMultiple       (); // @TBD

   bool               beginJob                  ();
   bool               newFrame                  ();
   bool               newFrame                  (char              *pszJobProperties);
   bool               endJob                    ();
   bool               abortJob                  ();

   bool               rasterize                 (PBYTE              pbBits,
                                                 PBITMAPINFO2       pbmi,
                                                 PRECTL             prectlPageLocation,
                                                 BITBLT_TYPE        eType);

   string            *queryCurrentJobProperties ();
   Enumeration       *listJobPropertyKeys       ();
   Enumeration       *listDeviceJobPropertyKeys ();
   string            *getJobPropertyType        (char              *pszKey);
   string            *getJobProperty            (char              *pszKey);

   Enumeration       *getDitherEnumeration      (); // @TBD

private:
   bool                 fAdvanced_d;

   int                  fdS2C_d;
   int                  fdC2S_d;
   char                *pszS2C_d;
   char                *pszC2S_d;

   int                  idBuffer1_d;
   int                  cbBuffer1_d;
   byte                *pbBuffer1_d;
   int                  idBuffer2_d;
   int                  cbBuffer2_d;
   byte                *pbBuffer2_d;

   PrinterCommand      *pCmd_d;

   char                *pszVersion_d;
   char                *pszDriverName_d;
   char                *pszDeviceName_d;
   char                *pszShortName_d;

   DeviceOrientation   *pDO_d;
   char                *pszDitherID_d;
   DeviceForm          *pDF_d;
   DeviceTray          *pDT_d;
   DeviceMedia         *pDM_d;
   DeviceResolution    *pDR_d;
   DevicePrintMode     *pDPM_d;
};

#endif
