/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _OmniProxy
#define _OmniProxy

#include "Device.hpp"
#include <stdio.h>

class OmniProxy : public Device
{
public:
   typedef struct _BitbltHeader {
      off_t       pos;                  // @TBD-VERIFY was: fpos_t      pos;
      RECTL       rectlPageLocation;
      int         cPlanes;
      int         cBitCount;
      int         cbBitmapHeader;
      int         cbBitmapData;
      BITBLT_TYPE eType;
   } BITBLT_HEADER, *PBITBLT_HEADER;

   typedef struct _BitbltHeaders {
      struct _BitbltHeaders *pNext;
      int                    cHeaders;
      BITBLT_HEADER          aHeaders[1];
   } BITBLT_HEADERS, *PBITBLT_HEADERS;

   #define BITBLT_HEADERS_BLOCK 4096
   #define BITBLT_HEADERS_COUNT ( ( BITBLT_HEADERS_BLOCK                    \
                                  - sizeof (((PBITBLT_HEADERS)0)->pNext)    \
                                  - sizeof (((PBITBLT_HEADERS)0)->cHeaders) \
                                  )                                         \
                                / sizeof (BITBLT_HEADERS_BLOCK)             \
                                )

                              OmniProxy                 (Device           *pDevice);
   virtual                   ~OmniProxy                 ();

   void                       initialize                ();

   char                      *getVersion                ();

   char                      *getDriverName             ();
   char                      *getDeviceName             ();
   char                      *getShortName              ();

   DeviceInstance            *getDeviceInstance         ();

   DeviceOrientation         *getCurrentOrientation     ();
   char                      *getCurrentDitherID        ();
   DeviceDither              *getCurrentDither          ();
   DeviceForm                *getCurrentForm            ();
   DeviceTray                *getCurrentTray            ();
   DeviceMedia               *getCurrentMedia           ();
   DeviceResolution          *getCurrentResolution      ();
   DevicePrintMode           *getCurrentPrintMode       ();
   DeviceCommand             *getCommands               ();
   DeviceData                *getDeviceData             ();
   DeviceGamma               *getCurrentGamma           ();

   int                        getPDLLevel               ();
   int                        getPDLSubLevel            ();
   int                        getPDLMajorRevisionLevel  ();
   int                        getPDLMinorRevisionLevel  ();

   bool                       hasCapability             (long              lMask);
   bool                       hasRasterCapability       (long              lMask);
   bool                       hasDeviceOption           (long              lMask);

   int                        getScanlineMultiple       ();

   bool                       beginJob                  ();
   bool                       beginJob                  (char             *pszJobProperties);
   bool                       newFrame                  ();
   bool                       newFrame                  (char             *pszJobProperties);
   bool                       endJob                    ();
   bool                       abortJob                  ();

   bool                       rasterize                 (PBYTE             pbBits,
                                                         PBITMAPINFO2      pbmi,
                                                         PRECTL            prectlPageLocation,
                                                         BITBLT_TYPE       eType);

   void                       setOutputStream           (ostream          *osNew);
   void                       setOutputStream           (int               iFileNo);
   void                       setOutputFunction         (PFNOUTPUTFUNCTION  pfnOutputFunction,
                                                         void              *pMagicCookie);

   bool                       sendBinaryDataToDevice    (BinaryData       *pData);
   bool                       sendBinaryDataToDevice    (DeviceForm       *pForm);
   bool                       sendBinaryDataToDevice    (DeviceTray       *pTray);
   bool                       sendBinaryDataToDevice    (DeviceMedia      *pMedia);
   bool                       sendBinaryDataToDevice    (DeviceResolution *pResolution);
   bool                       sendBinaryDataToDevice    (PBYTE             pbData,
                                                         int               iLength);

   bool                       sendPrintfToDevice        (BinaryData       *pData,
                                                                           ...);
   bool                       sendVPrintfToDevice       (BinaryData       *pData,
                                                         va_list           list);

   string                    *queryCurrentJobProperties ();
   Enumeration               *listJobPropertyKeys       ();
   Enumeration               *listDeviceJobPropertyKeys ();
   string                    *getJobPropertyType        (char             *pszKey);
   string                    *getJobProperty            (char             *pszKey);

   DeviceBlitter             *getDeviceBlitter          ();

   void                       loadLibrary               (char             *pszLibraryName);
   void                      *dlsym                     (char             *pszLibraryName,
                                                         char             *pszSymbol);

   string                     toString                  (ostrstream& oss);

private:
   void                      allocateBitmapInfo        ();
   void                      freeBitmapInfo            ();
   void                      replayBitmaps             ();

   Device          *pDevice_d;
   PBITBLT_HEADERS  pHeadersRoot_d;
   PBITBLT_HEADERS  pHeadersCurrent_d;
   PBITBLT_HEADER   pHeaderCurrent_d;
   FILE            *fp_d;
   int              fd_d;
   int              iMaxScanLines_d;
};

#endif
