/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#include "UPDFDomParser.hpp"
#include "MyErrorHandler.hpp"

#include <sys/stat.h>

const static bool fDebugOutput = false;

char *vpszExeName = 0;

int
main (int argc, char *argv[])
{
   bool         fValid;
   xmlValidCtxt validator = {
      &fValid,
      MyValidityErrorFunc,
      MyValidityWarningFunc
   };
   bool         fErrorEncountered = false;
   xmlNodePtr   nodeRootCommands  = 0;
   xmlNodePtr   nodeLocale        = 0;

   vpszExeName = argv[0];

   if (argc != 3)
   {
      cerr << "Usage: " << argv[0] << ": command_file locale_file < device_list" << endl;
      return 1;
   }

   xmlDocPtr docCommands = xmlParseFile (argv[1]);

   fValid = true;
   xmlValidateDocument (&validator, docCommands);

   if (!fValid)
   {
      cerr << "Error: \"" << argv[1] << "\" is not a valid xml file" << endl;
      return 1;
   }

   nodeRootCommands = xmlDocGetRootElement (docCommands);

   xmlDocPtr docLocale = xmlParseFile (argv[2]);

   fValid = true;
   xmlValidateDocument (&validator, docLocale);

   if (!fValid)
   {
      cerr << "Error: \"" << argv[2] << "\" is not a valid xml file" << endl;
      return 1;
   }

   nodeLocale = xmlDocGetRootElement (docLocale);

   char achLine[512]; // @TBD

   while (0 < cin.getline (achLine, sizeof (achLine)))
   {
//////cout << "line = \"" << achLine << "\"" << endl;

      if (  '#' != achLine[0]
         && sizeof (achLine) - 4 > strlen (achLine)
         )
      {
         strcat (achLine, ".xml");

         UPDFDomParser parser (achLine, nodeRootCommands, nodeLocale);

         if (parser.parse ())
         {
         }
         else
         {
            fErrorEncountered = true;
         }
      }
   }

   return fErrorEncountered;
}
