/*
 *   IBM Omni driver
 *   Copyright (c) International Business Machines Corp., 2000
 *
 *   This library is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as published
 *   by the Free Software Foundation; either version 2.1 of the License, or
 *   (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public License
 *   along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef _UPDFDomParser
#define _UPDFDomParser

#include <fstream.h>
#include <string>

#include <gnome-xml/parser.h>
#include <gnome-xml/tree.h>

#include <string>
#include <list>

typedef list <string> TrayList;
typedef list <string> FormList;
typedef list <string> MediaList;

class UPDFDomParser
{
public:
                UPDFDomParser             (const char  *pszXMLFile,
                                           xmlNodePtr   nodeRootCommands,
                                           xmlNodePtr   nodeLocale);
   virtual     ~UPDFDomParser             ();

   bool         parse                     ();
   void         setErrorCondition         (); // for ReportError interface

private:
   void         processForms              (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processMedias             (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processTrays              (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processResolutions        (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processOrientations       (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processConnections        (xmlNodePtr   node,
                                           ofstream    *pofstream);
   void         processData               (xmlNodePtr   rootNode,
                                           ofstream    *pofstream);
   void         processLocale             (xmlNodePtr   nodeLocale_d,
                                           ofstream    *pofstream);
   char        *findCommand               (char        *pszCmd,
                                           bool         fEvaluate);
   xmlNodePtr   findCommandNode           (char        *pszCmd);
   char        *findEventCommand          (char        *pszEventCommand,
                                           xmlNodePtr   nodeRasterGraphicList);

   xmlDocPtr   doc_d;
   string     *pstrXMLFile_d;
   bool        fErrorCondition_d;
   xmlNodePtr  nodeRootCommands_d;
   xmlNodePtr  nodeLocale_d;

   TrayList    listTrays_d;
   FormList    listForms_d;
   MediaList   listMedias_d;
};

#endif
