#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fstream.h>
#include <memory.h>
#include <glob.h>

#include <gnome-xml/parser.h>
#include <gnome-xml/tree.h>
#include <gnome-xml/xmlmemory.h>
#include <gnome-xml/valid.h>

char *
getElementType (xmlElementType type)
{
   switch (type)
   {
   case XML_ELEMENT_NODE:       return "XML_ELEMENT_NODE";
   case XML_ATTRIBUTE_NODE:     return "XML_ATTRIBUTE_NODE";
   case XML_TEXT_NODE:          return "XML_TEXT_NODE";
   case XML_CDATA_SECTION_NODE: return "XML_CDATA_SECTION_NODE";
   case XML_ENTITY_REF_NODE:    return "XML_ENTITY_REF_NODE";
   case XML_ENTITY_NODE:        return "XML_ENTITY_NODE";
   case XML_PI_NODE:            return "XML_PI_NODE";
   case XML_COMMENT_NODE:       return "XML_COMMENT_NODE";
   case XML_DOCUMENT_NODE:      return "XML_DOCUMENT_NODE";
   case XML_DOCUMENT_TYPE_NODE: return "XML_DOCUMENT_TYPE_NODE";
   case XML_DOCUMENT_FRAG_NODE: return "XML_DOCUMENT_FRAG_NODE";
   case XML_NOTATION_NODE:      return "XML_NOTATION_NODE";
   case XML_HTML_DOCUMENT_NODE: return "XML_HTML_DOCUMENT_NODE";
   default:                     return "???";
   }
}

char *
getAttributeType (xmlAttributeType type)
{
   switch (type)
   {
   case XML_ATTRIBUTE_CDATA:       return "XML_ATTRIBUTE_CDATA";
   case XML_ATTRIBUTE_ID:          return "XML_ATTRIBUTE_ID";
   case XML_ATTRIBUTE_IDREF:       return "XML_ATTRIBUTE_IDREF";
   case XML_ATTRIBUTE_IDREFS:      return "XML_ATTRIBUTE_IDREFS";
   case XML_ATTRIBUTE_ENTITY:      return "XML_ATTRIBUTE_ENTITY";
   case XML_ATTRIBUTE_ENTITIES:    return "XML_ATTRIBUTE_ENTITIES";
   case XML_ATTRIBUTE_NMTOKEN:     return "XML_ATTRIBUTE_NMTOKEN";
   case XML_ATTRIBUTE_NMTOKENS:    return "XML_ATTRIBUTE_NMTOKENS";
   case XML_ATTRIBUTE_ENUMERATION: return "XML_ATTRIBUTE_ENUMERATION";
   case XML_ATTRIBUTE_NOTATION:    return "XML_ATTRIBUTE_NOTATION";
   default:                        return "???";
   }
}

void
dumpNode (xmlNodePtr cur, int iLevel)
{
   static char achSpaces[] = "                                                                                            ";

   iLevel = strlen (achSpaces) - iLevel;

   printf ("%sname       = %s\n",   achSpaces+iLevel, cur->name);
   printf ("%stype       = %s\n",   achSpaces+iLevel, getElementType (cur->type));
   printf ("%schildren   = %08X\n", achSpaces+iLevel, (int)cur->childs);
   printf ("%snext       = %08X\n", achSpaces+iLevel, (int)cur->next);
   printf ("%sproperties = %08X\n", achSpaces+iLevel, (int)cur->properties);
   printf ("%scontent    = %s\n",   achSpaces+iLevel, cur->content);
}

void
printNode (xmlNodePtr cur)
{
   xmlNodePtr node;

   if (NULL == cur)
      return;

   node = cur;

   do
   {
      if (node->name)
        printf ("%s (%s)\n",
                getElementType (node->type),
                node->name);
      else
         printf ("%s (null)\n",
                 getElementType (node->type));

      xmlChar *buffer;

#ifndef XML_USE_BUFFER_CONTENT
      buffer = xmlEncodeEntitiesReentrant (node->doc, node->content);
#else
      buffer = xmlEncodeEntitiesReentrant (node->doc, xmlBufferContent (node->content));
#endif

      if (buffer)
      {
         printf("content: %s\n", buffer);

         xmlFree (buffer);
      }

      printf("hrm: %s\n", xmlNodeListGetString (node->doc, node->childs, 1));

      if (node->properties)
      {
         xmlAttrPtr attr = node->properties;

         do
         {
            printf ("property name (%s) type (%s)\n",
                    attr->name,
                    getElementType (attr->type));
            printf("value is\n");
            printf("-----\n");
            printNode (attr->val);
            printf("-----\n");

            attr = attr->next;

         } while (attr);

      }

      node = node->next;

   } while (node);

   if (cur->childs)
   {
      printNode (cur->childs);
   }
}

char *
bundleStringData (xmlNodePtr nodeItem)
{
   return (char *)xmlNodeListGetString (nodeItem->doc, nodeItem->childs, 1);
}

void
printNodeName (xmlNodePtr cur)
{
   const char *pszNodeName = 0;

   pszNodeName = bundleStringData (cur);

   cerr << "Node <"
        << cur->name
        << " "
        << pszNodeName
        << ">"
        << endl;

   if (pszNodeName)
   {
      free ((char *)pszNodeName);
   }
}

void
MyValidityErrorFunc (void *ctx, const char *msg, ...)
{
   va_list list;

   va_start (list, msg);

   printf ("Error: ");
   vprintf (msg, list);

   va_end (list);

   if (ctx)
      *((bool *)ctx) = false;
}

void
MyValidityWarningFunc (void *ctx, const char *msg, ...)
{
   va_list list;

   va_start (list, msg);

   printf ("Warning: ");
   vprintf (msg, list);

   va_end (list);
}

xmlNodePtr
nextNode (xmlNodePtr elm)
{
   do
   {
      // Move to the next node
      elm = elm->next;

   } while (  0 !=  elm
           && XML_COMMENT_NODE == elm->type
           );

   return elm;
}

bool
fixCommands (char      *pszFilename,
             xmlDocPtr  doc,
             xmlNodePtr deviceCommandsNode)
{
   xmlNodePtr elem;

   if (!deviceCommandsNode)
      return false;

   xmlNodePtr deviceCommandNode = deviceCommandsNode->childs;

   if (deviceCommandNode == 0)
      return false;

///while (deviceCommandNode != 0)
///{
///   dumpNode (deviceCommandNode, 0);
///   dumpNode (deviceCommandNode->childs, 3);
///
///   deviceCommandNode = deviceCommandNode->next;
///}

   // xmlNewNode (ns, name);
   // xmlNewChild (parent, ns, name, content);
   // xmlNodeSetName (node, name);
   // xmlSetProp (node, name, value);

   elem = xmlNewChild (deviceCommandsNode, NULL, (xmlChar *)"command", (xmlChar *)"_ESC_ \"2\"");
   xmlSetProp (elem, (xmlChar *)"name", (xmlChar *)"cmdSetSixthLineSpacing");

   elem = xmlNewChild (deviceCommandsNode, NULL, (xmlChar *)"command", (xmlChar *)"_ESC_ \"C%c\"");
   xmlSetProp (elem, (xmlChar *)"name", (xmlChar *)"cmdSetPageLengthInLines");

   FILE *fp = fopen (pszFilename, "w");

   printf ("fp = %08x\n", (int)fp);

   if (fp)
   {
      xmlDocDump (fp, doc);

      fclose (fp);
   }

   return true;
}

bool
fixDriver (char      *pszFilename,
           xmlDocPtr  doc,
           xmlNodePtr driverNode)
{
   const char *pszDriverName = 0;

   if (!driverNode)
      return false;

   xmlNodePtr nodeItem = driverNode->childs;

   if (nodeItem == 0)
      return false;

   pszDriverName = bundleStringData (nodeItem);

   // Move to the next node
   nodeItem = nextNode (nodeItem);
   if (nodeItem == NULL)
      return false;

   while (  NULL != nodeItem
         && 0 == strcmp ((char *)nodeItem->name, "Capability")
         )
   {
      // Move to the next node
      nodeItem = nextNode (nodeItem);
   }

   if (nodeItem == NULL)
      return false;

   while (  NULL != nodeItem
         && 0 == strcmp ((char *)nodeItem->name, "RasterCapabilities")
         )
   {
      // Move to the next node
      nodeItem = nextNode (nodeItem);
   }

   if (nodeItem == NULL)
      return false;

   while (  NULL != nodeItem
         && 0 == strcmp ((char *)nodeItem->name, "DeviceOptions")
         )
   {
      // Move to the next node
      nodeItem = nextNode (nodeItem);
   }

   if (nodeItem == NULL)
      return false;

   // Move to the next node
   nodeItem = nextNode (nodeItem);
   if (nodeItem == NULL)
      return false;

// printNodeName (nodeItem);

   xmlNodePtr nodeNew;
   xmlNodePtr nodeChild;
   xmlNodePtr nodeInsertTo;
   char       achText[512]; // @TBD

   sprintf (achText, "%s Orientations.xml", pszDriverName);

   nodeNew      = xmlNewNode (0, (const xmlChar *)"Has");
   nodeChild    = xmlNewText ((const xmlChar *)achText);
   nodeInsertTo = nodeItem;

   while (  NULL != nodeItem
         && (  0 == strcmp ((char *)nodeItem->name, "Uses")
            || 0 == strcmp ((char *)nodeItem->name, "Has")
            )
         )
   {
      const char *pszText = 0;

      pszText = bundleStringData (nodeItem);

      if (0 == strcmp (pszText, achText))
      {
         nodeInsertTo = 0;
      }

      // Move to the next node
      nodeItem = nextNode (nodeItem);

      if (pszText)
      {
         free ((char *)pszText);
      }
   }

   if (nodeInsertTo != NULL)
   {
      xmlAddChild (nodeNew, nodeChild);
      xmlAddPrevSibling (nodeInsertTo, nodeNew);

      xmlSaveFile (pszFilename, doc);
   }

   if (pszDriverName)
   {
      free ((char *)pszDriverName);
   }

   return true;
}

bool
fixForm (char      *pszFilename,
         xmlDocPtr  doc,
         xmlNodePtr formNodes)
{
   if (!formNodes)
      return false;

   xmlNodePtr formNode = formNodes->childs;

   if (formNode == 0)
      return false;

   while (formNode != 0)
   {
      xmlNodePtr nodeItem = formNode->childs;

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // Descend HCC
      nodeItem = nodeItem->childs;

      const char *pszNodeValue = 0;
      char        achValue[64];     // @TBD

      // hardCopyCapLeft
      pszNodeValue = bundleStringData (nodeItem);

      sprintf (achValue, "%d", atoi (pszNodeValue) * 10);

      xmlNodeSetContent (nodeItem->childs, (const xmlChar *)achValue);

      if (pszNodeValue)
      {
         free ((char *)pszNodeValue);
      }

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // hardCopyCapTop
      pszNodeValue = bundleStringData (nodeItem);

      sprintf (achValue, "%d", atoi (pszNodeValue) * 10);

      xmlNodeSetContent (nodeItem->childs, (const xmlChar *)achValue);

      if (pszNodeValue)
      {
         free ((char *)pszNodeValue);
      }

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // hardCopyCapRight
      pszNodeValue = bundleStringData (nodeItem);

      sprintf (achValue, "%d", atoi (pszNodeValue) * 10);

      xmlNodeSetContent (nodeItem->childs, (const xmlChar *)achValue);

      if (pszNodeValue)
      {
         free ((char *)pszNodeValue);
      }

      // Move to the next one
      nodeItem = nextNode (nodeItem);
      if (nodeItem == 0)
         return false;

      // hardCopyCapBottom
      pszNodeValue = bundleStringData (nodeItem);

      sprintf (achValue, "%d", atoi (pszNodeValue) * 10);

      xmlNodeSetContent (nodeItem->childs, (const xmlChar *)achValue);

      if (pszNodeValue)
      {
         free ((char *)pszNodeValue);
      }

      // Get the next deviceForm node
      formNode = nextNode (formNode);
   }

///xmlDocDump (stdout, doc);
   xmlSaveFile (pszFilename, doc);

   return true;
}

void
findAndFixCommands ()
{
   int          rc;
   glob_t       globbuf;
   xmlDocPtr    doc;
   xmlNodePtr   root;
   bool         fValid;
   char        *pszFilename = 0;
   xmlValidCtxt validator   = {
      &fValid,
      MyValidityErrorFunc,
      MyValidityWarningFunc
   };

   memset (&globbuf, 0, sizeof (globbuf));
   rc = glob ("* Commands.xml", 0, 0, &globbuf);

   if (0 == rc)
   {
      for (size_t i = 0; i < globbuf.gl_pathc; i++)
      {
         pszFilename = globbuf.gl_pathv[i];

         printf ("%s\n", pszFilename);

         doc = xmlParseFile (pszFilename);

         if (NULL == doc)
            return;

         fValid = true;
         xmlValidateDocument (&validator, doc);

         if (!fValid)
            return;

         root = xmlDocGetRootElement (doc);

         if (0 == strcmp ((char *)root->name, "deviceCommands"))
            fixCommands (pszFilename,
                         doc,
                         root);

         xmlFreeDoc (doc);
      }
   }

   globfree (&globbuf);
}

void
findAndFixDrivers ()
{
   int          rc;
   glob_t       globbuf;
   xmlDocPtr    doc;
   xmlNodePtr   root;
   bool         fValid;
   char        *pszFilename = 0;
   xmlValidCtxt validator   = {
      &fValid,
      MyValidityErrorFunc,
      MyValidityWarningFunc
   };

   memset (&globbuf, 0, sizeof (globbuf));
   rc = glob ("*.xml", 0, 0, &globbuf);

   if (0 == rc)
   {
      for (size_t i = 0; i < globbuf.gl_pathc; i++)
      {
         pszFilename = globbuf.gl_pathv[i];

         printf ("%s\n", pszFilename);

         doc = xmlParseFile (pszFilename);

         if (NULL == doc)
            return;

         fValid = true;
         xmlValidateDocument (&validator, doc);

         if (!fValid)
            return;

         root = xmlDocGetRootElement (doc);

         if (0 == strcmp ((char *)root->name, "Device"))
            fixDriver (pszFilename,
                       doc,
                       root);

         xmlFreeDoc (doc);
      }
   }

   globfree (&globbuf);
}

void
findAndFixForms ()
{
   int          rc;
   glob_t       globbuf;
   xmlDocPtr    doc;
   xmlNodePtr   root;
   bool         fValid;
   char        *pszFilename = 0;
   xmlValidCtxt validator   = {
      &fValid,
      MyValidityErrorFunc,
      MyValidityWarningFunc
   };

   memset (&globbuf, 0, sizeof (globbuf));
   rc = glob ("* Forms.xml", 0, 0, &globbuf);

   if (0 == rc)
   {
      for (size_t i = 0; i < globbuf.gl_pathc; i++)
      {
         pszFilename = globbuf.gl_pathv[i];

         printf ("%s\n", pszFilename);

         doc = xmlParseFile (pszFilename);

         if (NULL == doc)
            return;

         fValid = true;
         xmlValidateDocument (&validator, doc);

         if (!fValid)
            return;

         root = xmlDocGetRootElement (doc);

         if (0 == strcmp ((char *)root->name, "deviceForms"))
            fixForm (pszFilename,
                     doc,
                     root);

         xmlFreeDoc (doc);
      }
   }

   globfree (&globbuf);
}

int
main (int argc, char *argv[])
{
   findAndFixDrivers ();
   findAndFixForms ();

   return 0;
}
